@@ -25,11 +25,11 @@ int insert_gaps_new_align(char **seq, int site, int number, int numseqs,
 int reset_stars(SEA_VIEW *view, int debut, int lpart, char **seq, int lfrag, int *tmpselseqs);
 int confirm_refer_seq(int num_longest, SEA_VIEW *view, int *tmpselseqs);
 char *calc_msa_command(SEA_VIEW *view, char *base_fname);
-void cre_align_menu(SEA_VIEW *view, Fl_Menu_Button *obj);
+void cre_align_menu(SEA_VIEW *view);
 void align_opt_callback(Fl_Widget *ob, void *data);
 char *get_algo_opts(int algonum /* from 0 */, char **presname);
 char *get_opt_resname(int algonum);
-void set_algo_option_item(char *opts, Fl_Menu_Button *menu, int itemrank, int checkit);
+void set_algo_option_item(char *opts, Fl_Menu_Item *menu, int itemrank, int checkit);
 void add_align_callback(Fl_Widget *ob, void *data);
 void delete_align_callback(Fl_Widget *ob, void *data);
 void add_align_w_callback(Fl_Widget *ob, void *data);
@@ -60,9 +60,6 @@ extern void direct_help_callback(Fl_Widget *wgt, void *data);
 extern int compute_size_params(SEA_VIEW *view, int force_recompute);
 extern char **prepcolranks_by_codon(char **seq, int totseqs, int maxlen, int *eachlength, char **comments);
 extern int is_view_valid(SEA_VIEW *view);
-#ifdef WIN32
-extern int mysystem(const char *command);
-#endif
 
 
 /* extern variables */
@@ -586,10 +583,9 @@ sprintf(tmp, "msa_args_%d", view->alignment_algorithm +  1);
 strcpy(local, get_res_value(tmp, NULL));
 
 /* add user-entered options to command */
-Fl_Menu_Button *menu_align = (Fl_Menu_Button *)view->menu_align;
-int clustalopt = *(int *)menu_align->menu()->user_data();
+int clustalopt = ALIGN_OPTIONS + 1;
 optargs = get_algo_opts(view->alignment_algorithm, NULL);
-status = menu_align->mode(clustalopt + MAX_MSA_ALGOS );
+status = view->menu_align[clustalopt + MAX_MSA_ALGOS ].flags;
 if(optargs != NULL && (status & FL_MENU_VALUE) ) {
 	if((p=strchr(local, '>')) != NULL) {
 		*p = 0;
@@ -615,7 +611,6 @@ while((p = strstr(local, "%f")) != NULL) {
 /* build complete command string */
 #if defined(WIN32)
 	command = (char *)malloc(strlen(w32path) + strlen(local) + 30);
-//	sprintf(command, "cmd.exe /c \"%s\" %s ", w32path, local);
 	sprintf(command, "\"%s\" %s ", w32path, local);
 #else
 	command = (char *)malloc(strlen(get_full_path(algo_name)) + strlen(local) + 10);
@@ -628,13 +623,13 @@ return command;
 
 extern void fix_paste_timeout(void *u);
 
-void align_menu_callback(Fl_Widget *ob, void *unused)
+void align_menu_callback(Fl_Widget *ob, void *data)
 {
-	align_choice reponse = (align_choice) ((Fl_Menu_Button*)ob)->value();
-	SEA_VIEW *view = (SEA_VIEW *) ob->user_data();
+  SEA_VIEW *view = (SEA_VIEW *) ob->user_data();
+	int reponse = view->menubar->mvalue() - view->menu_align;
 	Fl::remove_timeout(fix_paste_timeout);
 	if(reponse <= ALIGN_SITES) { 
-	  align_selected_parts(view, view->alignment_algorithm, reponse );
+	  align_selected_parts(view, view->alignment_algorithm, (align_choice)reponse );
 	  if(is_view_valid(view)) {
 	    view->DNA_obj->redraw();
 	    view->horsli->redraw();
@@ -660,70 +655,90 @@ void align_menu_callback(Fl_Widget *ob, void *unused)
 }
 
 
-void cre_align_menu(SEA_VIEW *view, Fl_Menu_Button *obj)
+void cre_align_menu(SEA_VIEW *view)
 {
   char msa_name[60], *name; 
   int elt, attr;
-  static const int clustalopt = UNALIGN + 2;//keep to rank in menu of first alignment option
+  const int clustalopt = ALIGN_OPTIONS + 1;//keep to rank in menu of first alignment option
+  
+  static const Fl_Menu_Item alignitems_static[] = {
+    {"Align all", 0, align_menu_callback, 0, 0},
+    {"Align selected sequences", 0, align_menu_callback, 0, 0},
+    {"Align selected sites", 0, align_menu_callback, 0, 0},
+    {"Profile alignment", 0, align_menu_callback, 0, FL_MENU_DIVIDER},
+    {"De-align selection", 0, align_menu_callback, 0, FL_MENU_DIVIDER},
+    {"Alignment options", 0, align_menu_callback, 0, FL_SUBMENU},
+    
+    {"<none>", 0, NULL, 0, FL_MENU_TOGGLE | FL_MENU_INACTIVE},
+    {"Edit options", 0, edit_align_callback, 0, FL_MENU_DIVIDER},
+    {"Add external method", 0, add_align_w_callback, 0, 0},
+    {"Delete method", 0, delete_align_callback, 0, 0},
+    {0},
+    {0}
+  };
+  
+  view->alignment_algorithm = int_res_value("alignment", 1);
+  view->count_msa_algos = int_res_value("msa_algo_count", 0);
+
+  Fl_Menu_Item* alignitems = new Fl_Menu_Item[sizeof(alignitems_static)/sizeof(Fl_Menu_Item) + MAX_MSA_ALGOS];
+  memcpy(alignitems, alignitems_static, 6 * sizeof(Fl_Menu_Item));
+  memset(alignitems + 6, 0, MAX_MSA_ALGOS * sizeof(Fl_Menu_Item));
+  memcpy(alignitems + 6 + MAX_MSA_ALGOS, alignitems_static + 6, 6 * sizeof(Fl_Menu_Item));
 
-  ((Fl_Menu_Button*)obj)->add( "Align all", 0, align_menu_callback, (void *)&clustalopt, 0);
-  ((Fl_Menu_Button*)obj)->add( "Align selected sequences", 0, align_menu_callback, 0, 0);
-  ((Fl_Menu_Button*)obj)->add( "Align selected sites", 0, align_menu_callback, 0, 0);
-  ((Fl_Menu_Button*)obj)->add( "Profile alignment", 0, align_menu_callback, 0, FL_MENU_DIVIDER);
-  ((Fl_Menu_Button*)obj)->add( "De-align selection", 0, align_menu_callback, 0, FL_MENU_DIVIDER);
-  obj->add("Alignment options", 0, 0, 0, FL_SUBMENU);
   for(elt = 1; elt <= MAX_MSA_ALGOS; elt++) {
-    sprintf(msa_name, "Alignment options/hidden_%d", elt);
     attr = FL_MENU_RADIO; 
-    if(elt == view->count_msa_algos) attr |= FL_MENU_DIVIDER;
-    if(elt > view->count_msa_algos) attr |= FL_MENU_INVISIBLE;
-    attr = obj->add(msa_name, 0, align_opt_callback, (void *)&clustalopt, attr);
-    if(elt <= view->count_msa_algos) {
+    alignitems[clustalopt + elt - 1].callback(align_opt_callback, (void *)(elt - 1));
+    if (elt <= view->count_msa_algos) {
       sprintf(msa_name, "msa_name_%d", elt);
       name = get_res_value(msa_name, NULL);
-      obj->replace(clustalopt + elt - 1, name );
+      alignitems[clustalopt + elt - 1].label(strdup(name));
+      if (elt == view->count_msa_algos) attr |= FL_MENU_DIVIDER;
 #ifndef WIN32
       if(get_full_path(name) == NULL) {
-	attr = obj->mode(clustalopt + elt - 1);
-	obj->mode(clustalopt + elt - 1, attr | FL_MENU_INACTIVE);
+	attr = (alignitems + clustalopt + elt - 1)->flags;
+	attr |= FL_MENU_INACTIVE;
 	} 
 #endif
+      alignitems[clustalopt + elt - 1].flags = attr;
+      }
+    else {
+      alignitems[clustalopt + elt - 1].label("");
+      alignitems[clustalopt + elt - 1].flags = FL_MENU_INVISIBLE;
       }
     }
-  ((Fl_Menu_Item *)obj->menu() + clustalopt + view->alignment_algorithm)->set();
+  (alignitems + clustalopt + view->alignment_algorithm)->set();
   name = get_algo_opts(view->alignment_algorithm, NULL);
   char *options = new char[100 + (name ? strlen(name) : 0) ];
-  sprintf(options, "Alignment options/%s", name == NULL ? "<none>" : name);
+  sprintf(options, "%s", name == NULL ? "<none>" : name);
   attr = FL_MENU_TOGGLE;
   if(name == NULL) attr |= FL_MENU_INACTIVE;
-  obj->add(options, 0, NULL, 0, attr);
-  obj->add("Alignment options/Edit options", 0, edit_align_callback, (void *)&clustalopt, FL_MENU_DIVIDER);
-  obj->add("Alignment options/Add external method", 0, add_align_w_callback, (void *)&clustalopt, 0);
-  if(view->alignment_algorithm < 2) attr = FL_MENU_INACTIVE;
-  else attr = 0;
-  obj->add("Alignment options/Delete method", 0, delete_align_callback, (void *)&clustalopt, attr);
+  alignitems[clustalopt + MAX_MSA_ALGOS].label(strdup(options));
+  alignitems[clustalopt + MAX_MSA_ALGOS].flags = attr;
+  if (view->alignment_algorithm < 2) alignitems[clustalopt + MAX_MSA_ALGOS + 3].flags = FL_MENU_INACTIVE;
   delete options;
+  view->menu_align = alignitems;
+  view->menubar->add("Align", 0, NULL, (void*)view->menu_align, FL_SUBMENU_POINTER);
+  if (view->count_msa_algos >= MAX_MSA_ALGOS) (alignitems + clustalopt + MAX_MSA_ALGOS + 2)->deactivate();
 }
 
 
 void add_align_callback(Fl_Widget *ob, void *data)
 {
-if(data == NULL) {
+if(strcmp(ob->label(), "OK") != 0) {
 	ob->window()->hide();
 	return;
 	}
 char tmp[200];
 int doit = TRUE, x;
-int clustalopt = *(int *)data;
+int clustalopt = ALIGN_OPTIONS + 1;
 SEA_VIEW *view = (SEA_VIEW *)ob->window()->user_data();
-Fl_Menu_Button *menu_align = (Fl_Menu_Button *)view->menu_align;
+Fl_Menu_Item *menu_align = (Fl_Menu_Item *)view->menu_align;
 
 Fl_Window *w = ob->window();
-Fl_Widget **all = (Fl_Widget **)w->array();
-const char *name = ((Fl_Input *)all[0])->value();
-const char *args = ((Fl_Input *)all[1])->value();
+const char *name = ((Fl_Input *)w->child(0))->value();
+const char *args = ((Fl_Input *)w->child(1))->value();
 for(x = 0; x < view->count_msa_algos; x++) {
-	if(strcmp(name, ((Fl_Menu_Item *)menu_align->menu() + clustalopt + x)->label() ) == 0) doit = FALSE;
+	if(strcmp(name, (menu_align + clustalopt + x)->label() ) == 0) doit = FALSE;
 	}
 if(doit) doit = (name != NULL && args != NULL && strlen(name) >= 2 && strlen(args) >= 2);
 #ifdef WIN32
@@ -738,14 +753,14 @@ if(get_full_path(name) == NULL)
 	doit = FALSE;
 	}
 if(doit) {
-	menu_align->replace(clustalopt + view->count_msa_algos, name );
-	int stat = menu_align->mode(clustalopt + view->count_msa_algos - 1 );
-	menu_align->mode(clustalopt + view->count_msa_algos - 1, stat & ~FL_MENU_DIVIDER );
-	menu_align->mode(clustalopt + view->count_msa_algos, FL_MENU_RADIO | FL_MENU_DIVIDER);
+	menu_align[clustalopt + view->count_msa_algos].label(strdup(name));
+	int stat = menu_align[clustalopt + view->count_msa_algos - 1 ].flags;
+  menu_align[clustalopt + view->count_msa_algos - 1 ].flags = stat & ~FL_MENU_DIVIDER;
+  menu_align[clustalopt + view->count_msa_algos ].flags = FL_MENU_RADIO | FL_MENU_DIVIDER;
 	view->alignment_algorithm = view->count_msa_algos;
-	((Fl_Menu_Item *)menu_align->menu() + clustalopt + view->alignment_algorithm)->setonly();
+	(menu_align + clustalopt + view->alignment_algorithm)->setonly();
 	set_algo_option_item(NULL, menu_align, clustalopt + MAX_MSA_ALGOS, FALSE);
-	((Fl_Menu_Item *)menu_align->menu() + clustalopt + MAX_MSA_ALGOS + 3)->activate();
+	(menu_align + clustalopt + MAX_MSA_ALGOS + 3)->activate();
 	++(view->count_msa_algos);
 	sprintf(tmp, "%d", view->count_msa_algos);
 	set_res_value("msa_algo_count", tmp);
@@ -754,6 +769,7 @@ if(doit) {
 	sprintf(tmp, "msa_args_%d", view->count_msa_algos);
 	set_res_value(tmp, args);
 	save_resources();
+  if (view->count_msa_algos == MAX_MSA_ALGOS) (menu_align + clustalopt + MAX_MSA_ALGOS + 2)->deactivate();
 	}
 w->hide();
 }
@@ -761,9 +777,9 @@ w->hide();
 
 void delete_align_callback(Fl_Widget *ob, void *data)
 {
-int clustalopt = *(int*)data;
+int clustalopt = ALIGN_OPTIONS + 1;
 SEA_VIEW *view = (SEA_VIEW *)ob->user_data();
-Fl_Menu_Button *menu_align = (Fl_Menu_Button *)view->menu_align;
+Fl_Menu_Item *menu_align = (Fl_Menu_Item*)view->menu_align;
 char msa_name[20], *name;
 int num, status;
 
@@ -772,9 +788,10 @@ name = get_res_value(msa_name, NULL);
 if(name == NULL) return;
 int rep = fl_choice("Delete alignment method  '%s'  ?", "Keep", "Delete", NULL, name);
 if(rep == 0) return;
+  free((void*) (menu_align + clustalopt + view->alignment_algorithm)->label() );
 for(num = view->alignment_algorithm + 1; num <= view->count_msa_algos - 1; num++) {
-	name = (char *)((Fl_Menu_Item *)menu_align->menu() + clustalopt + num)->label();
-	((Fl_Menu_Item *)menu_align->menu() + clustalopt + num - 1)->label(name);
+	name = (char *)(menu_align + clustalopt + num)->label();
+	(menu_align + clustalopt + num - 1)->label(name);
 	
 	sprintf(msa_name, "msa_name_%d", num + 1);
 	name = get_res_value(msa_name, NULL);
@@ -796,10 +813,8 @@ for(num = view->alignment_algorithm + 1; num <= view->count_msa_algos - 1; num++
 	  }
 	else delete_res_value(get_opt_resname(num - 1));
 	}
-sprintf(msa_name, "hidden_%d", view->count_msa_algos);
-((Fl_Menu_Item *)menu_align->menu() + clustalopt + view->count_msa_algos - 1)->label(msa_name);
-status = menu_align->mode(clustalopt + view->count_msa_algos - 1 );
-menu_align->mode(clustalopt + view->count_msa_algos - 1, status | FL_MENU_INVISIBLE );
+  menu_align[clustalopt + view->count_msa_algos - 1].flags = FL_MENU_INVISIBLE;
+  menu_align[clustalopt + view->count_msa_algos - 1].label("");
 sprintf(msa_name, "msa_name_%d", view->count_msa_algos);
 delete_res_value(msa_name);
 sprintf(msa_name, "msa_args_%d", view->count_msa_algos);
@@ -809,16 +824,18 @@ delete_res_value(name);
 --(view->count_msa_algos);
 sprintf(msa_name, "%d", view->count_msa_algos);
 set_res_value("msa_algo_count", msa_name);
-status = menu_align->mode(clustalopt + view->count_msa_algos - 1 );
-menu_align->mode(clustalopt + view->count_msa_algos - 1, status | FL_MENU_DIVIDER );
-num = int_res_value("alignment", 1);
-view->alignment_algorithm = (num < view->count_msa_algos ? num : 1);
-((Fl_Menu_Item *)menu_align->menu() + clustalopt + view->alignment_algorithm)->setonly();
+status = menu_align[ clustalopt + view->count_msa_algos - 1 ].flags;
+menu_align[ clustalopt + view->count_msa_algos - 1 ].flags = status | FL_MENU_DIVIDER ;
+num = int_res_value("alignment", 0);
+view->alignment_algorithm = (num < view->count_msa_algos ? num : 0);
+  sprintf(msa_name, "%d", view->alignment_algorithm);
+  set_res_value("alignment", msa_name);
+(menu_align + clustalopt + view->alignment_algorithm)->setonly();
 name = get_algo_opts(view->alignment_algorithm, NULL);
 set_algo_option_item(name, menu_align, clustalopt + MAX_MSA_ALGOS, FALSE);
-num = menu_align->value();
-if(view->alignment_algorithm >= 2) ((Fl_Menu_Item *)menu_align->menu() + num)->activate();
-else ((Fl_Menu_Item *)menu_align->menu() + num)->deactivate();
+  Fl_Menu_Item* item = (Fl_Menu_Item*)view->menubar->mvalue();
+if (view->alignment_algorithm >= 2) item->activate();
+else item->deactivate();
 save_resources();
 }
 
@@ -831,7 +848,6 @@ chooser->title("Select external alignment program");
 #if defined(WIN32)
 	chooser->filter("*.exe");                 
 #endif
-//if ( chooser->show() != 0 ) return;
 char *filename = run_and_close_native_file_chooser(chooser);
 if(filename == NULL) return;
 ((Fl_Input*)data)->value(filename);
@@ -880,16 +896,16 @@ w->hotspot(w);
 void edit_align_callback(Fl_Widget *ob, void *data)
 {
 SEA_VIEW *view = (SEA_VIEW *)ob->user_data();
-Fl_Menu_Button *menu_align = (Fl_Menu_Button *)view->menu_align;
+Fl_Menu_Item *menu_align = (Fl_Menu_Item *)view->menu_align;
 char *opts, *optresname;
-char resname[20], name[50];
-int clustalopt = *(int*)data;
+char resname[20], *name;
+int clustalopt = ALIGN_OPTIONS + 1;
 
 sprintf(resname, "msa_name_%d", view->alignment_algorithm + 1);
-strcpy(name, get_res_value(resname, NULL) );
+name = get_res_value(resname, NULL);
 opts = get_algo_opts(view->alignment_algorithm, &optresname);
 opts = (char *)fl_input("Optional arguments for method   '%s'\n"
-"(Clustal ex: -gapopen=5 -quicktree -quiet | Muscle ex: -maxiters 2 -diags)",
+"(Clustal ex: --threads=4  | Muscle ex: -maxiters 2 -diags)",
 	opts, name);
 if(opts == NULL) return;
 if(strlen(opts) == 0) opts = NULL;
@@ -902,18 +918,17 @@ save_resources();
 
 void align_opt_callback(Fl_Widget *ob, void *data)
 {
-Fl_Menu_Button *menu_align = (Fl_Menu_Button *)ob;
-SEA_VIEW *view = (SEA_VIEW *)menu_align->user_data();
+SEA_VIEW *view = (SEA_VIEW *)ob->user_data();
 char *opts;
-int clustalopt = *(int*)data;
+int clustalopt = ALIGN_OPTIONS + 1;
 
-int reponse =  menu_align->value();
-if(reponse >= clustalopt && reponse < clustalopt + view->count_msa_algos) { 
-	view->alignment_algorithm = reponse - clustalopt;  
+int reponse =  view->menubar->mvalue() - view->menu_align - clustalopt;
+if(reponse >= 0 && reponse <  view->count_msa_algos) { 
+	view->alignment_algorithm = reponse;  
 	opts = get_algo_opts(view->alignment_algorithm, NULL);
-	set_algo_option_item(opts, menu_align, clustalopt + MAX_MSA_ALGOS, FALSE);
-	if(view->alignment_algorithm >= 2) ((Fl_Menu_Item *)menu_align->menu() + clustalopt + MAX_MSA_ALGOS + 3)->activate();
-	else ((Fl_Menu_Item *)menu_align->menu() + clustalopt + MAX_MSA_ALGOS + 3)->deactivate();
+	set_algo_option_item(opts, (Fl_Menu_Item*)view->menu_align, clustalopt + MAX_MSA_ALGOS, FALSE);
+	if(view->alignment_algorithm >= 2) ((Fl_Menu_Item *)view->menu_align + clustalopt + MAX_MSA_ALGOS + 3)->activate();
+	else ((Fl_Menu_Item *)view->menu_align + clustalopt + MAX_MSA_ALGOS + 3)->deactivate();
 	}
 }
 
@@ -934,40 +949,34 @@ return resname;
 }
 
 
-void set_algo_option_item(char *opts, Fl_Menu_Button *menu, int itemrank, int checkit)
+void set_algo_option_item(char *opts, Fl_Menu_Item *menu, int itemrank, int checkit)
 {
 int status;
 
 if(opts != NULL && strlen(opts) == 0) opts = NULL;
-menu->replace(itemrank, opts == NULL ? "<none>" : opts);
+  menu[itemrank].label(opts == NULL ? "<none>" : strdup(opts));
 status = FL_MENU_TOGGLE;
 if(opts == NULL) status |= FL_MENU_INACTIVE;
 else if(checkit) status |= FL_MENU_VALUE;
-menu->mode(itemrank, status );
+  menu[itemrank].flags = status;
 }
 
 
 void init_msa_algos(void)
 {
 int count = int_res_value("msa_algo_count", 0);
-#if defined(__APPLE__) || defined(WIN32) 
-	set_res_value("msa_name_1", "clustalw2"); 
-#else
-	set_res_value("msa_name_1", "clustalw"); 
-#endif
+set_res_value("msa_name_1", "clustalo"); 
+set_res_value("msa_args_1", "-v -i %f.pir -o %f.out --outfmt=fasta");
 if(count < 2) {
   set_res_value("msa_algo_count", "2");
-  set_res_value("alignment", "1"); /* muscle is default starting msa method */
-//set_res_value("msa_args_1", "-align -infile=%f.pir -outfile=%f.out -outorder=input -output=fasta");
-  set_res_value("msa_args_1", "-align -infile=%f.pir -outfile=%f.out -output=fasta");
+  set_res_value("alignment", "0"); /* clustal-omega is default starting msa method */
   set_res_value("msa_name_2", "muscle"); 
-//set_res_value("msa_args_2", "-in %f.pir -out %f.out -stable");
   set_res_value("msa_args_2", "-in %f.pir -out %f.out");
   }
 }
 
-enum {profile_clustalw2, profile_muscle};
-static int num_sp_set, use_quicktree, profile_to_profile = FALSE, profile_algo = profile_clustalw2,
+enum {profile_clustalo, profile_muscle};
+static int num_sp_set, profile_to_profile = FALSE, profile_algo = profile_clustalo,
   do_propagate;
 int profile_dialog(SEA_VIEW *view)
 // returns TRUE iff cancel
@@ -977,11 +986,14 @@ int profile_dialog(SEA_VIEW *view)
 	w->label("Profile alignment dialog");
 	y = 5;
 	Fl_Choice *choice = new Fl_Choice(155, y, 100, 20, "Profile sequences:");
-	for(i = 0; i < view->numb_species_sets; i++) choice->add(view->name_species_sets[i]);
+	for(i = 0; i < view->numb_species_sets; i++) {
+	  retval = choice->add("TeMpOrArY");
+	  choice->replace(retval, view->name_species_sets[i]);
+	  }
 	choice->value(0);
 	y += 25;
 	Fl_Choice *choice_algo = new Fl_Choice(155, y, 100, 20, "Alignment algorithm:");
-	choice_algo->add("clustalw2");
+	choice_algo->add("clustalo");
 	choice_algo->add("muscle");
 	choice_algo->value(profile_algo);
 	y = 75;
@@ -1003,11 +1015,7 @@ int profile_dialog(SEA_VIEW *view)
 	else if(profile_to_profile) profile->setonly();
 	else sequence->setonly();
 	y = group->y() + group->h() + 5;
-	Fl_Check_Button *quicktree = new Fl_Check_Button(5, y, 90, 20, "Quicktree");
-	quicktree->type(FL_TOGGLE_BUTTON);
-	if(profile_algo == profile_clustalw2) quicktree->value(1);
-	else quicktree->deactivate();
-  Fl_Check_Button *propagate = new Fl_Check_Button(quicktree->x() + quicktree->w(), y, 170, 20, 
+  Fl_Check_Button *propagate = new Fl_Check_Button(5, y, 170, 20, 
 						   "Propagate gaps to sites");
   propagate->type(FL_TOGGLE_BUTTON);
   if(view->regions == NULL && view->tot_comment_lines == 0) propagate->deactivate();
@@ -1033,18 +1041,14 @@ int profile_dialog(SEA_VIEW *view)
 			if(profile_algo == profile_muscle) {
 				profile->setonly();
 				sequence->deactivate();
-				quicktree->deactivate();
 				}
 			else {
 				sequence->activate();
 				sequence->setonly();
-				quicktree->activate();
-				quicktree->value(1);
 				}
 			}
 		}
 	num_sp_set = choice->value();
-	use_quicktree = quicktree->value();
   do_propagate = propagate->value();
 	profile_to_profile = profile->value();
 	delete w;
@@ -1101,7 +1105,7 @@ void profile_align(SEA_VIEW *view)
 		remove(fname2);
 		return;
 		}
-	p = get_res_value( profile_algo == profile_clustalw2 ? "msa_name_1" : "msa_name_2", NULL);
+	p = get_res_value( profile_algo == profile_clustalo ? "msa_name_1" : "msa_name_2", NULL);
 	strcpy(tmp, p);
 #ifdef WIN32
 	strcat(tmp, ".exe");
@@ -1113,12 +1117,13 @@ void profile_align(SEA_VIEW *view)
 		}
 	else {
 		command = new char[1000];
-		if(profile_algo == profile_clustalw2) {
-			sprintf(command, "\"%s\" %s -profile1=%s -profile2=%s %s -output=fasta -outorder=input"
-				" -outfile=%s.aligned -type=%s", p, 
-				profile_to_profile ? "-profile" : "-sequences",
-				fname1, fname2, 
-				use_quicktree ? "-quicktree" : "", base_fname,
+		if(profile_algo == profile_clustalo) {
+			sprintf(command, "\"%s\" -v --profile1=%s -%s%s  --outfmt=fasta "
+				" -o %s.aligned -t %s", p, 
+				fname1, 
+				profile_to_profile ? "-profile2=" : "i ",
+				fname2, 
+				base_fname,
 				view->protein ? "protein" : "dna" );
 			}
 		else { // muscle
@@ -1140,6 +1145,19 @@ void profile_align(SEA_VIEW *view)
 	/* lecture de l'alignement multiple */
 	if( status == 0 ) {
 		num = read_fasta_align(fname1, &seq, &seqname, &comments, &pheader, &err_message, FALSE);
+		if (num == total && !profile_to_profile) {
+		  // clustalo for "each seq to profile" outputs selected seqs first and profile second. We reverse this.
+		  for (i = 0; i < total; i++) {
+		    if (strncmp(seqname[i], "S_", 2) == 0) break;
+		    }
+		  if (i > 0) {
+		    char **seq2 = (char**)malloc(total * sizeof(char*));
+		    memcpy(seq2, seq+i, (total - i)*sizeof(char*));
+		    memcpy(seq2+(total-i), seq, i*sizeof(char*));
+		    memcpy(seq, seq2, total*sizeof(char*));
+		    free(seq2);
+		    }
+		  }
 		if(num > 0) { 	
 			for(i = 0; i < num; i++) free(seqname[i]);
 			free(seqname); free(comments); 
@@ -1158,13 +1176,13 @@ void profile_align(SEA_VIEW *view)
 	// compute table[rank in file xx.aligned] = rank in seaview alignment
 	table = (int *)calloc(total, sizeof(int));
 	rang = -1;
-	for(num = 0; num < view->tot_seqs; num++) {
+	for(num = 0; num < view->tot_seqs; num++) { // seqs from the reference profile
 		if(  ! view->list_species_sets[num_sp_set][num] ) continue;
 		rang++;
 		table[rang] = num;
 		back_to_stars(seq[rang], view->sequence[num]);
 	}
-	for(num = 0; num < view->tot_seqs; num++) {
+	for(num = 0; num < view->tot_seqs; num++) { // currently selected seqs
 		if(  ! view->sel_seqs[num] ) continue;
 		rang++;
 		table[rang] = num;
