@@ -364,8 +364,9 @@ raa_current_db->raa_sockfdw = fdopen(raa_snum,"a");
   	list_open_dbs_remove(raa_current_db);
  	return cantopensocket;
  	}
- reponse=read_sock(raa_current_db);
- if(reponse == NULL) {
+  // read first reply from the server
+  reponse = read_sock_timeout(raa_current_db, 1000*60 /* 1 min */);
+  if(reponse == NULL || strcmp(reponse, "OK acnuc socket started") != 0) {
   	list_open_dbs_remove(raa_current_db);
  	return cantopensocket;
  	}
