@@ -5,7 +5,7 @@
 #include <FL/Fl_Check_Button.H>
 #include <FL/Fl_Value_Input.H>
 #include <FL/Fl_Int_Input.H>
-
+#include <FL/Fl_Paged_Device.H>
 
 void custom_callback(Fl_Widget *obj, void *data);
 void my_colorchooser(Fl_Widget *obj, void *data);
@@ -28,7 +28,7 @@ static Fl_Input *protgroups, *s_protgroups, *movekeys;
 static Fl_Button **protcol;
 static Fl_Check_Button *lowercase, *inverted, *fastaspaces;
 static Fl_Int_Input *maxnamewidth;
-static Fl_Choice *pdfformat, *saveformat, *alignment_algo;
+static Fl_Choice *pdfformat, *saveformat, *alignment_algo, *fontsize;
 static Fl_Value_Input *pdffontsize;
 static Fl_Value_Input *phylipwidnames;
 
@@ -39,7 +39,7 @@ extern int numb_stdprotcolors, numb_altprotcolors;
 extern color_choice curr_color_choice;
 extern char *f_format_names[];
 extern int nbr_formats;
-extern paperformat printout_pageformat;
+extern Fl_Paged_Device::Page_Format printout_pageformat;
 extern int printout_fontsize;
 extern void mod_multipl(SEA_VIEW *view, int newval);
 extern void set_aa_color_mode(color_choice choice);
@@ -70,11 +70,12 @@ static Fl_Window *win = NULL;
 static Fl_Button *permanent, *accept;
 static coloraas *mycoloraas;
 int x, y, w, h, c;
-SEA_VIEW *view = (SEA_VIEW *)data;
+SEA_VIEW *view = (SEA_VIEW *)obj->user_data();
 Fl_Button *reset;
-
+  const Fl_Menu_Item *item;
+  
 if(win == NULL) {
-	win = new Fl_Window(310, 600, "Customization");
+	win = new Fl_Window(310, 640, "Customization");
 	
 	x = 9; y = 20; w = 160; h = 20;
 	bgrnd = new Fl_Slider(x,y,w,h, "background grey");
@@ -218,9 +219,18 @@ if(win == NULL) {
 	phylipwidnames = new Fl_Value_Input(x,y,w,h, "phylip names width");
 	phylipwidnames->align(FL_ALIGN_TOP);
 	phylipwidnames->labelsize(13);
+  
+  x = phylipwidnames->x();
+  y = phylipwidnames->y() + phylipwidnames->h() + 15;
+  fontsize = new Fl_Choice(x,y,w,h, "sequence font size");
+  fontsize->align(FL_ALIGN_TOP_LEFT);
+  item = view->menu_props;
+  while ((++item)->label()) {
+    fontsize->add(item->label());
+    }
 
 	x = A->x();
-	y += alignment_algo->h() + 10;
+	y += fontsize->h() + 10;
 	h = 40;
 	w = 95;
 	accept = new Fl_Button(x,y,w,h, "Apply");
@@ -286,19 +296,29 @@ lowercase->value(view->allow_lower);
 inverted->value(view->inverted_colors);
 fastaspaces->value(view->spaces_in_fasta_names);
 maxnamewidth->value(get_res_value("maxnamewidth", "20"));
-pdfformat->value(printout_pageformat);
+pdfformat->value(printout_pageformat == Fl_Paged_Device::A4 ? 0 : 1);
 pdffontsize->value(printout_fontsize);
 phylipwidnames->value(view->phylipwidnames);
 saveformat->value(view->format_for_save);
-Fl_Menu_Button *menu_align = (Fl_Menu_Button *)view->menu_align;
-int clustalopt = *(int *)menu_align->menu()->user_data();
+Fl_Menu_Item *menu_align = (Fl_Menu_Item *)view->menu_align;
+int clustalopt = ALIGN_OPTIONS + 1;
 alignment_algo->clear();
 for(x = 0; x < view->count_msa_algos; x++) {
-	Fl_Menu_Item *item = ((Fl_Menu_Item *)menu_align->menu() + clustalopt + x);
+	item = (menu_align + clustalopt + x);
 	alignment_algo->add( item->label() );
 	}	
 alignment_algo->value(view->alignment_algorithm);
-
+  fontsize->value(3);
+  item = view->menu_props;
+  while((++item)->label()) {
+    int fsize;
+    sscanf(item->label(), "%d", &fsize);
+    if (fsize == view->DNA_obj->labelsize()) {
+      fontsize->value(item - view->menu_props - 1);
+      break;
+      }
+  }
+  
 win->show();
 #ifndef MICRO
 win->hotspot(win);
@@ -318,7 +338,7 @@ void accept_cb(Fl_Widget *obj, void *data)
 coloraas *mycoloraas = (coloraas *)data;
 SEA_VIEW *view = (SEA_VIEW *)mycoloraas->user_data();
 if(!is_view_valid(view)) return;
-Fl_Menu_Button *menu_props = (Fl_Menu_Button *)view->bouton_props;
+  Fl_Menu_Item *menu_props = (Fl_Menu_Item *)view->menu_props;
 props_menu_parts *props_parts = (props_menu_parts *)menu_props->user_data();
 char *p;
 Fl_Menu_Item *item;
@@ -347,15 +367,14 @@ curr_color_choice = prep_aa_color_code((char *)protgroups->value(), (char *)s_pr
 	&numb_stdprotcolors, &numb_altprotcolors);
 view->numb_stdprotcolors = numb_stdprotcolors;
 view->numb_altprotcolors = numb_altprotcolors;
-int etat = menu_props->mode(props_parts->colors + 2);
-if(view->numb_altprotcolors > 1) menu_props->mode(props_parts->colors + 2, etat & ~FL_MENU_INACTIVE);
-else menu_props->mode(props_parts->colors + 2, etat | FL_MENU_INACTIVE);
+  if (view->numb_altprotcolors > 1) (menu_props + props_parts->colors + 2)->activate();
+  else (menu_props + props_parts->colors + 2)->deactivate();
 if(view->protein) {
 	view->alt_colors = (color_choice)-1; /* to force redraw of sequences */
-	menu_props->value(props_parts->colors + 1);
-	item = (Fl_Menu_Item *)menu_props->menu() + props_parts->colors;
-	item->do_callback(menu_props);
-	(item+1)->setonly();
+	item = menu_props + props_parts->colors + 1;
+  item->setonly();
+  view->menubar->value(item);
+  item->callback()(view->menubar, props_parts);
 	if(view->numb_altprotcolors > 1) view->alt_colors = USING_STANDARD_COLORS;
 	else view->alt_colors = NO_ALT_COLORS;
 	}
@@ -367,16 +386,15 @@ else {
 	view->multipl->redraw();
 	}
 
-item = (Fl_Menu_Item *)menu_props->menu() + props_parts->allow_lower;
-view->allow_lower = ! lowercase->value(); /* because callback toggles it */
-item->do_callback(menu_props);
+item = menu_props + props_parts->allow_lower;
+  view->allow_lower = lowercase->value();
 if(view->allow_lower) item->set();
 else item->clear();
 
-item = (Fl_Menu_Item *)menu_props->menu() + props_parts->inverted;
-view->inverted_colors = ! inverted->value(); /* because callback toggles it */
-menu_props->value(props_parts->inverted);
-item->do_callback(menu_props);
+item = menu_props + props_parts->inverted;
+view->inverted_colors =! inverted->value(); /* because callback toggles it */
+  view->menubar->value(item);
+  item->callback()(view->menubar, (void*)props_parts->inverted);
 if(view->inverted_colors) item->set();
 else item->clear();
   
@@ -384,7 +402,7 @@ view->spaces_in_fasta_names = fastaspaces->value();
   
 set_res_value("maxnamewidth", ((Fl_Input*)maxnamewidth)->value());
 
-printout_pageformat = (paperformat)pdfformat->value();
+printout_pageformat = pdfformat->value() == 0 ? Fl_Paged_Device::A4 : Fl_Paged_Device::LETTER;
 printout_fontsize = (int)pdffontsize->value();
 view->phylipwidnames = (int)phylipwidnames->value();
 if(view->phylipwidnames < 10) {
@@ -395,10 +413,15 @@ if(view->phylipwidnames < 10) {
 view->format_for_save = (known_format)saveformat->value();
 
 view->alignment_algorithm = alignment_algo->value();
-Fl_Menu_Button *menu_align = (Fl_Menu_Button *)view->menu_align;
-int clustalopt = *(int *)menu_align->menu()->user_data();
-item = (Fl_Menu_Item *)menu_align->menu() + clustalopt;
+Fl_Menu_Item *menu_align = (Fl_Menu_Item *)view->menu_align;
+int clustalopt = ALIGN_OPTIONS + 1;
+item = menu_align + clustalopt;
 (item + view->alignment_algorithm)->setonly();
+  
+  item = (Fl_Menu_Item*)view->menu_props + fontsize->value() + 1;
+  item->setonly();
+  view->menubar->value(item);
+  item->callback()(view->menubar, NULL);
 
 if(!view->protein) view->DNA_obj->redraw();
 fl_reset_cursor(view->dnawin);
@@ -446,7 +469,7 @@ set_res_value("movekeys", movekeys->value() );
 set_res_value("lowercase", lowercase->value() ? "true" : "false");
 set_res_value("inverted", inverted->value() ? "true" : "false");
 set_res_value("spacesinfastanames", fastaspaces->value() ? "true" : "false");
-set_res_value("printoutpageformat", printout_pageformat == A4 ? "A4" : "LETTER" );
+set_res_value("printoutpageformat", printout_pageformat == Fl_Paged_Device::A4 ? "A4" : "LETTER" );
 sprintf(txt, "%d", (int)pdffontsize->value() );
 set_res_value("printoutfontsize",  txt);
 
@@ -468,6 +491,8 @@ for(i = 0; i < 21; i++) {
 	}
 *(p-1) = 0;
 set_res_value("codoncolors", txt);
+  
+set_res_value("sequencefontsize", fontsize->text());
 
 save_resources();
 obj->window()->hide();
@@ -625,7 +650,7 @@ int colorcell::handle(int event)
 				}
 			mychooser->selected = this;
 			this->redraw();
-			this->take_focus();
+			Fl::focus(this);
 			break;
 		case FL_KEYBOARD:
 			if( Fl::event_state(FL_COMMAND) ) return 0; // can be shortcut of other widgets
