@@ -762,7 +762,7 @@ char *replace_ranks_by_names(char *tree, int ntaxa, char **taxnames)
 {
   char *p = tree, *q, *r;
   char *newtree;
-  char bootstrap_comment[300];
+  char bootstrap_comment[50];
   int rank, l;
   float pp;
   
@@ -782,7 +782,11 @@ char *replace_ranks_by_names(char *tree, int ntaxa, char **taxnames)
     if (*p == ')' && *(p+1) == '[') { // a comment with a PP value
       *q++ = *p++;
       r = bootstrap_comment;
-      do *r++ = *p++; while (*p != ']');
+      do {
+	*r = *p++;
+	if (r - bootstrap_comment < sizeof(bootstrap_comment) - 1) r++;
+      }
+      while (*p != ']');
       p++;
       *r = 0;
       if ( strncmp(bootstrap_comment, "[&prob=", 7) == 0) {
@@ -1149,8 +1153,8 @@ int read_nexus_align(char *fname, char ***pseqs, char ***pseqnames,
 	int **comment_length, int *protein, SEA_VIEW *view)
 {
 FILE *in;
-int lseqs, i, tot;
-char **tree_names;
+int lseqs, i, tot, tot_trees = 0;
+char **tree_names, **trees = NULL;
 
 if( (in = is_nexus_file(fname) ) == NULL) {
 	*err_message = (char *)"not a NEXUS file";
@@ -1160,18 +1164,23 @@ tot = read_nexus_file(in, pseqs, &lseqs, protein, pseqnames,
 	pcomments, pheader,
 	pnum_species_sets, list_species_sets, name_species_sets,
 	charsets, ptot_comment_lines, comment_name, comment_line,
-	&view->tot_trees, &view->trees, &tree_names);
-Fl_Menu_Button *menu_trees = (Fl_Menu_Button *)view->menu_trees;
-for(i = 0; i < view->tot_trees; i++) {
-	int rank = menu_trees->add(tree_names[i]);
-	((Fl_Menu_Item *)menu_trees->menu() + rank)->labelfont(FL_HELVETICA_ITALIC);
-	free(tree_names[i]);
-	}
-if(view->tot_trees > 0) free(tree_names);
-if(tot == 0 && view->tot_trees == 0) {
-	*err_message = (char *)"format error";
-	return 0;
-	}
+	&tot_trees, &trees, &tree_names);
+if(tot == 0 && tot_trees == 0) {
+  *err_message = (char *)"format error";
+  return 0;
+  }
+for(i = 0; i < tot_trees; i++) {
+  view->menu_trees->add(tree_names[i], trees_callback, NULL, 0);
+  int rank = view->menu_trees->vlength();
+  (view->menu_trees->vitem(rank - 1))->labelfont(FL_HELVETICA_ITALIC);
+  free(tree_names[i]);
+  }
+if(tot_trees > 0) {
+  free(tree_names);
+  view->trees = (char**)realloc(view->trees, (view->tot_trees + tot_trees) * sizeof(char*));
+  memcpy(view->trees + view->tot_trees, trees, tot_trees * sizeof(char*));
+  }
+view->tot_trees += tot_trees;
 if(*pcomments == NULL) {
 	*pcomments = (char **)calloc(tot, sizeof(char *));
 	}
