@@ -2,6 +2,12 @@
 #include <FL/Fl_Round_Button.H>
 #include <FL/Fl_Check_Button.H>
 #include <ctype.h>
+#ifdef WIN32
+#include <wchar.h>
+extern "C" {
+  int _wsystem(const wchar_t *command );
+}
+#endif
 
 
 enum position_mask {custom_mask = -1, gblocks_sites, position_1, position_2, position_3, position_12,
@@ -37,13 +43,8 @@ static void insert_in_active_region(SEA_VIEW *view, int numsite, int total);
 void insert_region_part(SEA_VIEW *view, int numsite, int total);
 list_regions *parse_regions_from_header(char *header);
 void draw_region_background(SEA_VIEW *view, int f_seq0, int l_seq0);
-const char *create_species_set(SEA_VIEW *view, char *set_name);
+const char *create_species_set(SEA_VIEW *view, const char *set_name);
 void species_menu_callback(Fl_Widget *ob, void *extra);
-void get_menu_taille_valeur(Fl_Widget *menu, int *taille, int *valeur);
-void init_menu_taille_valeur(Fl_Widget *menu, int taille, int valeur, 
-	int offset);
-void update_menu_taille_valeur(Fl_Widget *menu, int taille, int valeur, 
-	int offset);
 int parse_species_sets_from_header(char *header, int **list_species_sets,
 	char **name_species_sets, int totseqs);
 
@@ -154,7 +155,7 @@ else	{
 	}
 view->active_region = duplicate_region(maregion, name);
 if(view->active_region == NULL) return TRUE; /*mess no mem serait mieux */
-Fl_Menu_Item *items = (Fl_Menu_Item *) ((Fl_Menu_ *)view->menu_file)->menu();
+Fl_Menu_Item *items = (Fl_Menu_Item *)view->menu_file;
 items[SAVE_REGIONS].activate();
 view->region_length = view->seq_length;
 memset(view->region_line, 'X', view->region_length);
@@ -196,6 +197,7 @@ void create_variable_mask(SEA_VIEW *view, region *maregion)
     }
 }
 
+
 int create_gblocks_mask(SEA_VIEW *view, region *maregion)
 /* return 0 iff OK */
 {
@@ -273,10 +275,17 @@ int create_gblocks_mask(SEA_VIEW *view, region *maregion)
   if (b4_val) strcat(cmd, " -b4=5");
   if (b3_val) strcat(cmd, " -b3=4");
   if (b2_val) sprintf(cmd + strlen(cmd), " -b2=%d", view->tot_seqs/2);
-#ifndef WIN32
+#ifdef WIN32
+  //conversion of utf-8 cmd into a WCHAR string
+  int wlen = MultiByteToWideChar(CP_UTF8, 0, cmd, -1, NULL, 0);
+  WCHAR *wcmd = new WCHAR[wlen];
+  MultiByteToWideChar(CP_UTF8, 0, cmd, -1, wcmd, wlen);  
+  _wsystem(wcmd);
+  delete[] wcmd;
+#else
   strcat(cmd, " > /dev/null");
-#endif
   system(cmd);
+#endif
   sprintf(fname, "%s.fst-gb.txts", base_fname);
   in = fopen(fname, "r");
   if (in == NULL) status = 1;
@@ -363,7 +372,7 @@ int create_codon_position_mask(SEA_VIEW *view, position_mask mask, const char *r
   if (old != NULL) {
     if (mask != gblocks_sites) return 2;
     int taille, valeur;
-    get_menu_taille_valeur(view->menu_regions, &taille, &valeur);
+    taille = view->menu_sites->vlength();
     valeur = taille;
     while(old->next) {
       old = old->next;
@@ -419,7 +428,7 @@ int create_codon_position_mask(SEA_VIEW *view, position_mask mask, const char *r
   }
   view->active_region = duplicate_region(maregion, maregion->name);
   if(view->active_region == NULL) return TRUE; /*mess no mem serait mieux */
-  Fl_Menu_Item *items = (Fl_Menu_Item *) ((Fl_Menu_ *)view->menu_file)->menu();
+  Fl_Menu_Item *items = (Fl_Menu_Item *)view->menu_file;
   items[SAVE_REGIONS].activate();
   compute_size_params(view, TRUE);
   return 1;  
@@ -459,7 +468,7 @@ else	{
 	while(old->next != NULL) old = old->next;
 	old->next = new_list;
 	}
-Fl_Menu_Item *items = (Fl_Menu_Item *) ((Fl_Menu_ *)view->menu_file)->menu();
+Fl_Menu_Item *items = (Fl_Menu_Item *)view->menu_file;
 items[SAVE_REGIONS].activate();
 return TRUE;
 }
@@ -471,9 +480,10 @@ list_regions *list;
 list_segments *act_seg, *mem_seg;
 int tmp, taille;
 
-if(view->menu_regions == NULL) return;
-get_menu_taille_valeur(view->menu_regions, &taille, &tmp);
-if(tmp == 0) return;
+if(view->menu_sites == NULL) return;
+  tmp = view->menu_sites->value();
+  if(tmp == 0) return;
+  taille = view->menu_sites->vlength();
 list = view->regions;
 if(tmp > 1) while( --tmp) list= list->next;
 /* compare active and region in list */
@@ -499,7 +509,7 @@ void hide_region_line(SEA_VIEW *view)
 {
 free_region(view->active_region);
 view->active_region = NULL;
-Fl_Menu_Item *items = (Fl_Menu_Item *) ((Fl_Menu_ *)view->menu_file)->menu();
+Fl_Menu_Item *items = (Fl_Menu_Item *)view->menu_file;
 items[SAVE_REGIONS].deactivate();
 compute_size_params(view, TRUE);
 }
@@ -510,11 +520,8 @@ void delete_region(SEA_VIEW *view, int rang)
 list_regions *list, *previous;
 region *maregion;
 list_segments *segment, *next;
-int taille, valeur;
 
-  get_menu_taille_valeur(view->menu_regions, &taille, &valeur);
-  ((Fl_Menu_ *)view->menu_regions)->remove(rang + 4 - 1);
-  update_menu_taille_valeur(view->menu_regions, taille - 1, 0, 4);
+  view->menu_sites->remove(rang-1);
 list = view->regions;
 if(rang == 1) 
 	view->regions = list->next;
@@ -548,8 +555,7 @@ while ( --rang )
 	list = list->next;
 if(view->viewasprots && list->element->hide_when_viewasprots) return;
 if(view->active_region == NULL) {
-	Fl_Menu_Item *items = 
-		(Fl_Menu_Item *) ((Fl_Menu_ *)view->menu_file)->menu();
+	Fl_Menu_Item *items = (Fl_Menu_Item *)view->menu_file;
 	items[SAVE_REGIONS].activate();
 	}
 view->active_region = duplicate_region(list->element, list->element->name);
@@ -724,13 +730,13 @@ void regions_menu_callback(Fl_Widget *ob, void *extra)
   SEA_VIEW *view;
   const char *reg_name;
   position_mask mask;
-  int taille, valeur, ok, reponse = ((Fl_Menu_ *)ob)->value() + 1;
-  Fl_Menu_Item *items = (Fl_Menu_Item *) ((Fl_Menu_ *)ob)->menu();
+  Fl_Menu_ *menu = (Fl_Menu_*)ob;
+  view = (SEA_VIEW *) ob->user_data();
+  int reponse = menu->mvalue() - view->menu_sites->get_menu();
+  int ok;
+  Fl_Menu_Item *items = view->menu_sites->get_menu();
 
-  if(reponse <= 0 ) return;
-  view = (SEA_VIEW *) ((user_data_plus *)extra)->p;
-  get_menu_taille_valeur(ob, &taille, &valeur);
-  if(reponse == 1) { /* create new region */
+  if(reponse == 0) { /* create new region */
 	reg_name = site_mask_creation_dialog(view, &mask);
 	if(reg_name == NULL) return;
 	if(mask == custom_mask) ok = create_full_region(view, reg_name);
@@ -748,12 +754,10 @@ void regions_menu_callback(Fl_Widget *ob, void *extra)
 	items[1].activate();
 	items[2].activate();
 	items[3].activate();
-	get_menu_taille_valeur(ob, &taille, &valeur);
-	((Fl_Menu_ *)ob)->add(reg_name, 0, NULL, NULL, FL_MENU_RADIO);
-	taille++;
-	update_menu_taille_valeur(ob, taille, taille, 4);
+    view->menu_sites->add(reg_name, regions_menu_callback, NULL, FL_MENU_RADIO);
+    view->menu_sites->value(view->menu_sites->vlength());
 	}
-  else if(reponse == 2) { /* duplicate current region */
+  else if(reponse == 1) { /* duplicate current region */
 	  reg_name = fl_input("New set name?", view->active_region->name);
 	  if(reg_name == NULL) return;
 	  if( !rename_current_region(view, reg_name) ) {
@@ -761,11 +765,10 @@ void regions_menu_callback(Fl_Widget *ob, void *extra)
 		  return;
 		  }
 	  view->DNA_obj->redraw();
-	  ((Fl_Menu_ *)ob)->add(reg_name, 0, NULL, NULL, FL_MENU_RADIO);
-	  taille++;
-	  update_menu_taille_valeur(ob, taille, taille, 4);
+    view->menu_sites->add(reg_name, regions_menu_callback, NULL, FL_MENU_RADIO);
+    view->menu_sites->value(view->menu_sites->vlength());
 	  }
-  else if(reponse == 3) { /* hide current region */
+  else if(reponse == 2) { /* hide current region */
 	  save_active_region(view);
 	  hide_region_line(view);
 	  view->DNA_obj->redraw();
@@ -773,16 +776,19 @@ void regions_menu_callback(Fl_Widget *ob, void *extra)
 	  items[1].deactivate();
 	  items[2].deactivate();
 	  items[3].deactivate();
-	  update_menu_taille_valeur(ob, taille, 0, 4);
+    view->menu_sites->value(0);
 	  }
-  else if(reponse == 4) { /* delete current region */
+  else if(reponse == 3) { /* delete current region */
 	  list_regions *list;
-	  int tmp;
+	  int valeur, tmp;
+    valeur = view->menu_sites->value();
+    if (!valeur) return;
 	  tmp = valeur;
 	  list = view->regions;
 	  if(tmp > 1) while( --tmp) list= list->next;
 	  if( fl_choice("Confirm deletion of set:\n%s", "Cancel", "Delete", NULL, list->element->name) ) {
 		  delete_region(view, valeur);
+	    view->menu_sites->value(0);
 		  view->DNA_obj->redraw();
 		  items[0].activate();
 		  items[1].deactivate();
@@ -792,11 +798,11 @@ void regions_menu_callback(Fl_Widget *ob, void *extra)
 	  }
   else	{ /* choix d'une region courante */
 	  save_active_region(view);
-	  valeur = reponse - 4;
+	  int valeur = menu->mvalue() - view->menu_sites->vitem(0) + 1;
 	  activate_region_line(view, valeur);
 	  compute_size_params(view, TRUE);
 	  view->DNA_obj->redraw();
-	  update_menu_taille_valeur(ob, taille, valeur, 4);
+    view->menu_sites->value(valeur);
 	  items[0].deactivate();
 	  items[1].activate();
 	  items[2].activate();
@@ -1191,7 +1197,7 @@ while( (segment = segment->next) != NULL );
 }
 
 
-const char *create_species_set(SEA_VIEW *view, char *set_name)
+const char *create_species_set(SEA_VIEW *view, const char *set_name)
 {
 int newnumb, i;
 if(view->numb_species_sets >= MAX_SPECIES_SETS)
@@ -1220,14 +1226,16 @@ return NULL;
 void species_menu_callback(Fl_Widget *ob, void *extra)
 {
 SEA_VIEW *view;
-char *set_name;
+  Fl_Menu_* menu = (Fl_Menu_*)ob;
+const char *set_name;
 const char *message;
-int i, rang, numset, taille, reponse = ((Fl_Menu_ *)ob)->value() + 1;
+int i, rang, numset, taille;
 
-if(reponse <= 0 ) return;
-view = (SEA_VIEW *) ((user_data_plus *)extra)->p;
-get_menu_taille_valeur(ob, &taille, &rang);
-if(reponse == 1) { /* create new set of species */
+view = (SEA_VIEW *)ob->user_data();
+int reponse = menu->mvalue() - view->menu_species->get_menu();
+  taille = view->menu_species->vlength();
+  rang = view->menu_species->value();
+if (reponse == 0) { /* create new set of species */
 	if(view->tot_sel_seqs == 0) return;
 	set_name = (char *)fl_input("New species set name?", "good seqs");
 	if(set_name == NULL) return;
@@ -1235,20 +1243,18 @@ if(reponse == 1) { /* create new set of species */
 		fl_alert("%s\n%s", message, set_name);
 		return;
 		}
-	((Fl_Menu_ *)ob)->add(set_name, 0, NULL, NULL, FL_MENU_RADIO);
-	taille++;
-	update_menu_taille_valeur(ob, taille, taille, 2);
-	Fl_Menu_Item *items = (Fl_Menu_Item *) ((Fl_Menu_ *)ob)->menu();
-	items[0].deactivate();
-	items[1].activate();
+  view->menu_species->add(set_name, species_menu_callback, NULL, FL_MENU_TOGGLE);
+  view->menu_species->value(++taille);
+  view->menu_species->get_menu()->deactivate();
+  (view->menu_species->get_menu()+1)->activate();
 	}
-else if( reponse == 2) { /* effacer un set */
-	Fl_Menu_Item *items = (Fl_Menu_Item *) ((Fl_Menu_ *)ob)->menu();
+else if(reponse == 1) { /* effacer un set */
+  Fl_Menu_Item *items = view->menu_species->get_menu();
 	items[1].deactivate();
 	if(rang == 0) return; /* par securite */
 	numset = rang - 1;
-	((Fl_Menu_ *)view->menu_species)->remove(numset + 3 - 1);
-	init_menu_taille_valeur(view->menu_species, taille - 1, 0, 2);
+  view->menu_species->remove(numset);
+	items[0].deactivate();
 	free(view->list_species_sets[numset]);
 	free(view->name_species_sets[numset]);
 	memcpy(view->list_species_sets + numset,
@@ -1264,7 +1270,7 @@ else if( reponse == 2) { /* effacer un set */
 	}
 else	{ /* choix d'un set courant */
 	int minvis, maxvis, visible = FALSE, premier = 0;
-	rang = reponse - 2;
+  rang = menu->mvalue() - view->menu_species->vitem(0) + 1;
 	memcpy(view->sel_seqs, view->list_species_sets[rang-1], 
 		view->tot_seqs * sizeof(int) );
 	view->tot_sel_seqs = 0;
@@ -1283,55 +1289,79 @@ else	{ /* choix d'un set courant */
 		((Fl_Slider *)view->vertsli)->value(premier);
 		}
 	view->DNA_obj->redraw();
-	update_menu_taille_valeur(ob, taille, rang, 2);
-	Fl_Menu_Item *items = (Fl_Menu_Item *) ((Fl_Menu_ *)ob)->menu();
-	items[1].activate();
+  view->menu_species->value(rang);
+  (view->menu_species->get_menu()+1)->activate();
 	adjust_menu_edit_modes(view);
 	}
 }
 
 
-void get_menu_taille_valeur(Fl_Widget *menu, int *taille, int *valeur)
+vlength_menu::vlength_menu(Fl_Menu_ *m, const char *name, const Fl_Menu_Item *init, int clength)
 {
-user_data_plus *extra = (user_data_plus *)menu->user_data();
-int u_ldata = extra->value;
-*taille = u_ldata / 100;
-*valeur = u_ldata % 100;
+  menu_ = m;
+  offset = clength;
+  vlen = 0;
+  val = 0;
+  items = (Fl_Menu_Item*)malloc((offset+1)*sizeof(Fl_Menu_Item));
+  memcpy(items, init, offset*sizeof(Fl_Menu_Item));
+  memset(items + offset, 0, sizeof(Fl_Menu_Item));
+  rank = menu_->add(name, 0, NULL, items, FL_SUBMENU_POINTER);
+}
+
+void vlength_menu::add(const char *txt, Fl_Callback *cb, void *data, int flags)
+{
+  items = (Fl_Menu_Item*)realloc(items, (offset + vlen + 2)*sizeof(Fl_Menu_Item));
+  ((Fl_Menu_Item*)(menu_->menu() + rank))->user_data( items );
+  memset(items + offset + vlen, 0, 2*sizeof(Fl_Menu_Item));
+  (items + offset + vlen)->label( strdup(txt) );
+  (items + offset + vlen)->callback(cb, data);
+  (items + offset + vlen)->flags = flags;
+  vlen++;
+}
+
+void vlength_menu::remove(int rank)
+{
+  if (rank >= vlen) return;
+  free( (void*)vitem(rank)->label() );
+  memmove(items + offset + rank, items + offset + rank + 1, (vlen - rank)*sizeof(Fl_Menu_Item));
+  vlen--;
 }
 
 
-void init_menu_taille_valeur(Fl_Widget *obj, int taille, int valeur, 
-	int offset)
+Fl_Menu_Item* vlength_menu::find_item(const char* txt)
 {
-int i;
-Fl_Menu_ *menu = (Fl_Menu_ *)obj;
-
-for(i = 1; i <= taille; i++) menu->mode(offset + i - 1, FL_MENU_RADIO);
-if(valeur != 0) menu->mode(offset + valeur - 1, FL_MENU_RADIO | FL_MENU_VALUE);
-int u_ldata = 100 * taille + valeur;
-user_data_plus *extra = (user_data_plus *)menu->user_data();
-extra->value = u_ldata;
+  for (int i = 0; i < vlen; i++) {
+    if (strcmp(vitem(i)->label(), txt) == 0) return vitem(i);
+    }
+  return NULL;
 }
 
-
-void update_menu_taille_valeur(Fl_Widget *obj, int taille, int valeur, 
-	int offset)
+void vlength_menu::value(int v) 
 {
-int old_taille, old_valeur;
-Fl_Menu_ *menu = (Fl_Menu_ *)obj;
-
-get_menu_taille_valeur(menu, &old_taille, &old_valeur);
-if(valeur != old_valeur) {
-	if(old_valeur != 0) 
-		menu->mode(offset + old_valeur - 1, FL_MENU_RADIO);
-	if(valeur != 0) 
-		menu->mode(offset + valeur - 1, FL_MENU_RADIO | FL_MENU_VALUE);
-	}
-int u_ldata = 100 * taille + valeur;
-user_data_plus *extra = (user_data_plus *)menu->user_data();
-extra->value = u_ldata;
+  if (v) vitem(v - 1)->setonly();
+  else if (val) vitem(val - 1)->clear();
+  val = v;
 }
 
+void vlength_menu::vclear() 
+{
+  for (int i = 0; i < vlen; i++) {
+    free( (void*)(items+offset+i)->label() );
+  }
+  vlen = 0;
+  val = 0;
+  memset(items + offset, 0, sizeof(Fl_Menu_Item));
+}
+
+vlength_menu::~vlength_menu()
+{
+  for (int i = 0; i < vlen; i++) {
+    free( (void*)(items+offset+i)->label() );
+    }
+  free(items);
+  ((Fl_Menu_Item*)(menu_->menu() + rank))->user_data( NULL );
+  ((Fl_Menu_Item*)(menu_->menu() + rank))->flags = 0;
+}
 
 int parse_species_sets_from_header(char *header, int **list_species_sets,
 	char **name_species_sets, int totseqs)
