@@ -4,18 +4,21 @@
 #include <stdlib.h>
 #include <string.h>
 #include <limits.h>
-#include "FL/Fl.H"
-#include "FL/Fl_Window.H"
-#include "FL/Fl_Menu_Item.H"
-#include "FL/Fl_Menu_Button.H"
-#include "FL/Fl_Browser.H"
-#include "FL/Fl_Input.H"
-#include "FL/Fl_Box.H"
-#include "FL/Fl_Return_Button.H"
-#include "FL/Fl_Repeat_Button.H"
-#include "FL/Fl_Choice.H"
-#include "FL/fl_draw.H"
-#include "FL/fl_ask.H"
+#include <FL/Fl.H>
+#if 100*FL_MAJOR_VERSION + FL_MINOR_VERSION < 103
+#error Seaview requires FLTK version 1.3.0 or higher
+#endif
+#include <FL/Fl_Window.H>
+#include <FL/Fl_Menu_Item.H>
+#include <FL/Fl_Menu_Bar.H>
+#include <FL/Fl_Browser.H>
+#include <FL/Fl_Input.H>
+#include <FL/Fl_Box.H>
+#include <FL/Fl_Return_Button.H>
+#include <FL/Fl_Repeat_Button.H>
+#include <FL/Fl_Choice.H>
+#include <FL/fl_draw.H>
+#include <FL/fl_ask.H>
 #include <FL/Fl_Native_File_Chooser.H>
 #ifndef FALSE
 #define FALSE 0
@@ -46,16 +49,11 @@ extern char *my_fgets(char *s, int n, FILE *f);
 
 #ifdef WIN32
 #include <wchar.h>
-#include <pdflib.h>
 typedef const wchar_t *LPCWSTR;
 extern char *wchartoutf8(LPCWSTR in);
 extern LPCWSTR utf8towchar(const char *in);
 extern void my_bell(void); 
 #define fopen(a,b) _wfopen(utf8towchar(a), L##b)
-extern int win32_PDF_begin_document(PDF *p, const char *filename, int len, const char *optlist);
-extern void win32_PDF_end_document(PDF *p, const char *optlist);
-#define PDF_begin_document win32_PDF_begin_document
-#define PDF_end_document win32_PDF_end_document
 #define memccpy _memccpy
 #define fl_ringbell(a) my_bell()
 #elif defined __APPLE__
@@ -70,7 +68,12 @@ extern void fl_ringbell(int);
 #define PREPARE_LABEL(p) mac_fname_to_roman(p)
 extern void set_seaview_modified(struct _SEA_VIEW *view, int ismodified);
 #else
+#if defined(WIN32)
+extern const char *win32_UTF8_to_CP1252(const char *in); 
+#define PREPARE_LABEL(p) win32_UTF8_to_CP1252(p)
+#else
 #define PREPARE_LABEL(p) p
+#endif
 #define set_seaview_modified(view, ismodif) view->modif_but_not_saved = ismodif
 #endif
 typedef struct _user_data_plus {
@@ -115,7 +118,7 @@ typedef enum {MASE_FORMAT, PHYLIP_FORMAT, CLUSTAL_FORMAT, MSF_FORMAT,
 
 typedef enum {OPEN_ANY, OPEN_MASE, OPEN_PHYLIP, OPEN_CLUSTAL, OPEN_MSF, OPEN_FASTA,
 	OPEN_NEXUS, ACNUC_IMPORT, SAVE, SAVE_AS, SAVE_REGIONS, SAVE_PROT_ALIGN, PRINTOUT, 
-#ifndef __APPLE__
+#ifdef WIN32
 	PDFOPTIONS,
 #endif
 	CONCATENATE, NEW_WINDOW, CLOSE_WINDOW,QUIT
@@ -130,15 +133,36 @@ typedef enum {COPY_SEQS, PASTE_SEQS, SELECT_ALL, RENAME_SEQ, EDIT_COMMENTS, EDIT
 	CONSENSUS_SEQ, DELETE_GAP_ONLY_SITES, SET_GCODE}
 	edit_choice;
 
-typedef enum { ALIGN_ALL, ALIGN_SELECTED_SEQS, ALIGN_SITES, PROFILE, UNALIGN } align_choice;
+typedef enum { ALIGN_ALL, ALIGN_SELECTED_SEQS, ALIGN_SITES, PROFILE, UNALIGN, ALIGN_OPTIONS } align_choice;
 
 typedef enum {SHOW_HIDE_FOOTERS, CREATE_FOOTER, DELETE_FOOTER }
 	footers_menu_choice;
 	
-typedef enum {A4, LETTER} paperformat;
-
 typedef enum {PDF_COLOR, PDF_BW, TEXT_ONLY} pdf_kind;
 
+class vlength_menu // an FL_SUBMENU_POINTER-attached variable-length series of Fl_Menu_Item's
+{
+  int offset; // # of fixed items
+  int vlen; // # of variable items
+  Fl_Menu_Item *items; // array of fixed + variable menu items
+  Fl_Menu_ *menu_; // menu of which this is a submenu
+  int rank; // rank of menu item to which this is a submenu
+  int val; // rank (from 1) of currently selected variable item
+public:
+  vlength_menu(Fl_Menu_ *m, const char *name, const Fl_Menu_Item *init, int clength);
+  int vlength() {return vlen;} // lenth of variable part
+  int value() { return val; } // 0 if unset, >=1 if one if set
+  void value(int v); // 0 to clear, >=1 to set one
+  Fl_Menu_Item *get_menu() {return items;} // fixed + variable parts
+  void add(const char *txt, Fl_Callback *cb, void *data, int flags); // add at end of variable part
+  Fl_Menu_Item* vitem(int r) { return items + offset + r; } // one of the variable items
+  Fl_Menu_Item* bar_item() { return (Fl_Menu_Item*)menu_->menu() + rank; }
+  void remove(int rank); // remove variable item of rank (from 0)
+  Fl_Menu_Item* find_item(const char*txt);
+  void vclear(); // remove all the variable part
+  ~vlength_menu();
+};
+
 #define TREE_WINDOW "Tree"
 #define SEAVIEW_WINDOW "Seaview"
 
@@ -149,15 +173,15 @@ typedef struct _SEA_VIEW {
   Fl_Widget *right_screen_move, *left_screen_move, *up_screen_move, 
 		  *down_screen_move; /* 4 big jump arrows */
   Fl_Widget *multipl; /* box displaying the multiplier value */
-  Fl_Widget *menu_regions; /* menu of region operations */
-  Fl_Widget *menu_file; /* menu of file operations */
-  Fl_Widget *menu_edit; /* menu of edit operations */
-  Fl_Widget *menu_align; /* menu of align operations */
-  Fl_Widget *bouton_props; /* menu of properties */
-  Fl_Widget *menu_species; /* menu of species */
-  Fl_Widget *menu_footers; /* menu of footers */
-  Fl_Widget *menu_trees; /* menu of trees */
-  int offset_to_local_trees; /* first local tree in menu_trees */
+  Fl_Menu_Bar *menubar;
+  const Fl_Menu_Item *menu_file; /* menu items of file operations */
+  const Fl_Menu_Item *menu_edit; /* menu items of edit operations */
+  const Fl_Menu_Item *menu_align; /* menu of align operations */
+  const Fl_Menu_Item *menu_props; /* menu of properties */
+  vlength_menu *menu_sites; /* menu of site operations */
+  vlength_menu *menu_species; /* menu of species */
+  const Fl_Menu_Item *menu_footers; /* menu of footers */
+  vlength_menu *menu_trees; /* menu of trees */
   Fl_Widget *bouton_reference; /* bouton_reference */
   char **sequence; /* array of sequences */
   char **comments; /* array of sequence comments */
@@ -243,9 +267,8 @@ typedef struct _SEA_VIEW {
 	
 
 typedef struct _props_menu_parts {
-	int fontsize, colors, viewasprots, inverted, edit, reference, keys,
-		consensusopt, slow_fast, allow_lower, custom, stats;
-	SEA_VIEW *view;
+	int colors, viewasprots, inverted, edit, reference, keys,
+		consensusopt, slow_fast, allow_lower;
 	int usingcodoncolors;
 	} props_menu_parts;
 
@@ -260,7 +283,7 @@ extern void draw_region_line(Fl_Widget *ob, SEA_VIEW *view);
 extern void props_button_callback(Fl_Widget *ob, long mainpop);
 extern void regions_menu_callback(Fl_Widget *ob, void *extra);
 extern void species_menu_callback(Fl_Widget *ob, void *extra);
-extern void footers_menu_callback(Fl_Widget *ob, long which);
+extern void footers_menu_callback(Fl_Widget *ob, void *extra);
 extern int suppr_segment(region *maregion, int site, char *line);
 extern int begin_change_segment(SEA_VIEW *view, int new_site);
 extern int continue_change_segment(SEA_VIEW *view, int new_site);
@@ -300,11 +323,6 @@ extern char *save_alignment_or_region(const char *fname, char **seq,
 extern Fl_Window *load_alignment_file(SEA_VIEW *view, char *filename, 
 	const char *message, known_format file_format, int doing_dnd);
 extern int is_a_protein_seq(char *seq);
-extern int prepare_printout(const char *filename, char **seq, int totseqs,  
-	char **seqname,
-	int *eachlength, int char_per_line,
-	int block_size, int lines_per_page, int printout_vary,
-	char *align_name, int ref0);
 extern char *get_full_path(const char *fname);
 extern void draw_region_background(SEA_VIEW *view, int f_seq0, int l_seq0);
 extern void load_seq_dialog(SEA_VIEW *view);
@@ -334,6 +352,7 @@ extern int insert_gap_all_comments(int numgaps, int pos,  SEA_VIEW *view);
 extern int delete_in_all_comments(int numdels, int pos,  SEA_VIEW *view);
 extern char* fl_file_chooser(const char* message, const char* pat, const char* fname, int);
 extern char *run_and_close_native_file_chooser(Fl_Native_File_Chooser *chooser, int keepalive=0);
+extern void trees_callback(Fl_Widget *ob, void *data);
 #ifdef __APPLE__
 extern char *mac_fname_to_roman(const char *in); 
 #endif
