@@ -1,4 +1,3 @@
-
 #include "seaview.h"
 #include <ctype.h>
 #include <FL/Fl_Multiline_Output.H>
@@ -9,22 +8,7 @@ char aalist[30] = "ACDEFGHIKLMNPQRSTVWY";
 int TRANSVONLY=0;
 char infotext[10000];
 
-/* check_alloc */
-/* Return a pointer with the required allocated memory or leave program */
-/* if not enough memory. */
-
 extern void *check_alloc(int nbrelt, int sizelt);
-/*
- void *check_alloc(int nbrelt, int sizelt)
-{
-  void *retval; 
-  
-  if(nbrelt == 0) return NULL;
-  if( (retval=(char*)calloc(nbrelt,sizelt)) != NULL ) { return retval; }
-  printf("Not enough memory\n");
-  exit(0);
-}
-*/
 
 void bydefault_aagroups(){
   int i;
@@ -361,6 +345,7 @@ int freq_obs(char* seq1, char* seq2, double* freq){
 
 static void close_win_callback(Fl_Widget *ob, void *data)
 {
+  if (!ob->as_window()) ob = ob->window();
   ob->hide();
   Fl::delete_widget( ob );
 }
@@ -579,9 +564,11 @@ void geni_act(SEA_VIEW *view)
   ml->textfont(FL_COURIER);
   ml->user_data(view->masename);
   win->resizable(ml);
-  Fl_Button *b = new Fl_Button(win->w()/2 - 25 , win->h() - 20, 50, 20, "Save");
+  Fl_Button *b = new Fl_Button(win->w()/2 - 25 , win->h() - 23, 50, 20, "Save");
   b->callback(save_stat_cb, ml);
-  
+  Fl_Return_Button *ok = new Fl_Return_Button(win->w() - 55 , win->h() - 23, 50, 20, "OK");
+  ok->callback(close_win_callback);
+
   if(view->tot_sel_seqs == 0) sprintf(text, "%d species\n", nb);
   else {
     sprintf(text, "%d selected species\n", nb);
@@ -705,5 +692,5 @@ void geni_act(SEA_VIEW *view)
 
 void stats_callback(Fl_Widget *obj, void *data)
 {
-  geni_act((SEA_VIEW*)data);
+  geni_act((SEA_VIEW*)obj->user_data());
 }
