@@ -1,10 +1,5 @@
-#include "FL/Fl.H"
-#include "FL/fl_draw.H"
-#include "FL/Enumerations.H"
-#include <FL/names.h>
 #include "treedraw.h"
 #include "unrooted.h"
-#if 100*FL_MAJOR_VERSION + FL_MINOR_VERSION >= 103
 #include "svg.h"
 
 extern void do_plot(FD_nj_plot *fd_nj_plot, int doing_print);
@@ -174,7 +169,6 @@ void svg_tree_save(FD_nj_plot *fd_nj_plot)
     fd_unrooted->y = fd_unrooted->y_offset = 0;
     fd_unrooted->w = fd_nj_plot->panel->w();
     fd_unrooted->h = fd_nj_plot->panel->h();
-    fd_unrooted->pdf = NULL;
     frame_and_draw_unrooted(fd_unrooted);
     fd_unrooted->x_offset = old_x;
     fd_unrooted->y_offset = old_y;
@@ -185,5 +179,3 @@ void svg_tree_save(FD_nj_plot *fd_nj_plot)
   fclose(out);
 }
 
-#endif // 100*FL_MAJOR_VERSION + FL_MINOR_VERSION
-
