@@ -6,12 +6,11 @@
 #include <FL/Fl_Round_Button.H>
 #include <FL/Fl_Check_Button.H>
 #include <FL/Fl_Select_Browser.H>
-#include <FL/Fl_Menu_Button.H>
 #include <FL/Fl_Return_Button.H>
 #include <FL/Fl_Int_Input.H>
 #include <FL/Fl_Float_Input.H>
-#include "FL/Fl.H"
-#include "FL/fl_ask.H"
+#include <FL/Fl.H>
+#include <FL/fl_ask.H>
 
 extern "C" {
 #include "phyml_util.h"
@@ -26,15 +25,16 @@ void free_after_view_to_allseq(allseq *phyml_seqs);
 void initialize_random_series(void);
 void bootstrap_weights(allseq *seqs, int use_codons);
 void savedistinfile(matrix *mat, const char *dirname);
-matrix *calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs, int in_bootstrap);
+matrix *calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs, int in_bootstrap, char **p_err_mess);
 void phyml_dialog(SEA_VIEW *view);
 static void change_searching_callback(Fl_Widget *ob, void *data);
 static void cancel_callback(Fl_Widget *ob, void *data);
 static void model_callback(Fl_Widget *ob, void *data);
 void run_phyml_callback(Fl_Widget *ob, void *data);
 void distance_method_dialog(SEA_VIEW *view);
+matrix *run_calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs, int in_bootstrap, char **p_err_mess);
 void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int use_bionj, int use_bootstrap,
-						 int replicates, int *interrupted, Fl_Box *box, const char *distance_name, int using_kaks,
+						 int replicates, Fl_Box *box, const char *distance_name, int using_kaks,
 						 int savetofile, const char *user_tree);
 char *bootstrap_reformat(char *tree, int replicates);
 matrix *Obs_Dist(allseq *data, model *mod);
@@ -48,9 +48,9 @@ int informative(char** sequ, int nb);
 void jumble_fct(char** seq, char** name, int notu);
 void parsimony_dialog(SEA_VIEW *view);
 char *parsimony_with_jumble(char **pars_seqs, char **pars_names, int n_otu, int njumbles, int protein, int *psteps,
-	int *pcount, int maxbest);
+	int *pcount, int maxbest, Fl_Widget *w_count);
 char *parsimony_with_bootstrap(allseq *tmpallseq, char **pars_seqs, char **pars_names, int n_otu, int njumbles, 
-	int protein, int *psteps, int maxbest, int replicates, Fl_Box *w_count, int *pinterrupted, int *preplicates_done);
+	int protein, int *psteps, int maxbest, int replicates, Fl_Box *w_count, int *preplicates_done);
 double *vector(long nl, long nh);
 void free_vector(double *v, long nl, long nh);
 int ludcmp(double **a, int n, int *indx, double *d);
@@ -61,6 +61,7 @@ char *put_names_back_in_tree(char *oldtree, char **names);
 char *replace_tree_names_by_rank(const char *tree, char **names, int notu);
 
 
+extern void interrupt_callback(Fl_Widget *ob, void *data);
 extern int save_phylip_file(const char *fname, char **seq,
 					 char **seqname, int totseqs, int *eachlength, region *region_used, 
 					 int *sel_seqs, int tot_sel_seqs, int phylipwidnames);
@@ -77,23 +78,26 @@ extern int save_resources(void);
 extern int set_res_value(const char *name, const char *value);
 extern char *create_tmp_filename(void);
 extern int is_view_valid(SEA_VIEW *view);
+char *parsimony(char** seq, char** seqname, int notu, int* steps, char* toevaluate, int arg_maxtrees, int *bt_weights, bool);
+extern int fl_create_thread(void (*f) (void *), void* p, unsigned);
 extern "C" {
-	char *dnapars(char** seq, char** seqname, int notu, int* steps, char* toevaluate, int arg_maxtrees, int *bt_weights);
-	char *protpars(char** seq, char** seqname, int notu, int* steps, char* toevaluate, int arg_maxtrees,
-				   int *bt_weights);
-	 void prefastlwl(void);
-	 void loadrl(void);
-	 int fastlwl(char **seq, int nbseq, int lgseq, double **ka, double **ks, 
-						double **vka, double **vks, int* sat1, int* sat2, int *wght);
-	}
+  void prefastlwl(void);
+  void loadrl(void);
+  int fastlwl(char **seq, int nbseq, int lgseq, double **ka, double **ks, 
+					double **vka, double **vks, int* sat1, int* sat2, int *wght);
+  char* dnapars(char** seq, char** seqname, int notu, int *steps, char* toevaluate, int arg_maxtrees, int *bt_weights); 
+  char* protpars(char** seq, char** seqname, int notu, int *steps, char* toevaluate, int arg_maxtrees, int *bt_weights); 
+}
 
+int tree_build_interrupted;
 
 void trees_callback(Fl_Widget *ob, void *data)
 /* 0:Distance  1:PhyML 2:import
  */
 {
-	SEA_VIEW *view = (SEA_VIEW *)data;
-	int reponse = ((Fl_Menu_Button*)ob)->value();
+	SEA_VIEW *view = (SEA_VIEW *)ob->user_data();
+	//int reponse = (int)data;
+  int reponse = view->menubar->mvalue() - view->menu_trees->get_menu();
 	if(reponse <= 2) {
 		if(view->tot_seqs < 3 || (view->tot_sel_seqs > 0 && view->tot_sel_seqs < 3) ) {
 			fl_alert("Can't draw tree for less than 3 sequences");
@@ -140,8 +144,9 @@ void trees_callback(Fl_Widget *ob, void *data)
 	else if(reponse == 4) {//New empty tree window
 		treedraw(NULL, view, NULL, FALSE);
 		}
-	else if(reponse >= view->offset_to_local_trees) {
-		char *p = view->trees[reponse - view->offset_to_local_trees];
+	else {
+	  reponse = view->menubar->mvalue() - view->menu_trees->vitem(0);
+		char *p = view->trees[reponse];
 		Fl_Window *w = Fl::first_window();
 		while(w != NULL) {
 			const char *c = w->xclass();
@@ -153,14 +158,14 @@ void trees_callback(Fl_Widget *ob, void *data)
 			w = Fl::next_window(w);
 			}
 		char *tree = strdup(p);
-		treedraw(tree, view, ((Fl_Menu_Button*)ob)->mvalue()->label(), TRUE);
+		treedraw(tree, view, ((Fl_Menu_*)ob)->mvalue()->label(), TRUE);
 		}
 }
 
-static void interrupt_callback(Fl_Widget *wgt, void *data)
+/*static void interrupt_callback(Fl_Widget *wgt, void *data)
 {
 	*(int *)data = TRUE;
-}
+}*/
 
 static void distance_callback(Fl_Widget *wgt, void *data)
 {
@@ -186,7 +191,7 @@ void distance_method_dialog(SEA_VIEW *view)
 	static Fl_Check_Button *bootstrap;
 	static Fl_Button *interrupt;
 	static Fl_Return_Button *go;
-	static int interrupted, started, maxchoice;
+	static int started, maxchoice;
 	if(first) {
 		first = FALSE;
 		w = new Fl_Window(250, 220);
@@ -201,7 +206,7 @@ void distance_method_dialog(SEA_VIEW *view)
 		distance = new Fl_Choice(90, nj->y() + nj->h() + 15, 100, 20, "Distance");
 		distance->align(FL_ALIGN_LEFT);
 		ignore_all_gaps = new Fl_Check_Button(160, distance->y() + distance->h() + 5, 50, 20, 
-															   "ignore all gap sites");
+						      "ignore all gap sites");
 		ignore_all_gaps->value(1);
 		ignore_all_gaps->align(FL_ALIGN_LEFT);
 		int y = ignore_all_gaps->y() + ignore_all_gaps->h() + 25;
@@ -219,11 +224,11 @@ void distance_method_dialog(SEA_VIEW *view)
 		bionj->setonly();
 	
 		interrupt = new Fl_Button(3, w->h() - 25, 70, 20, "");
-		interrupt->callback(interrupt_callback, &interrupted);
+		interrupt->callback(interrupt_callback, &tree_build_interrupted);
 		go = new Fl_Return_Button(w->w() - 70 - 3, interrupt->y() , 70, 20, "Go");
 		go->callback(interrupt_callback, &started);
 		w->end();
-		w->callback(interrupt_callback, &interrupted);
+		w->callback(interrupt_callback, &tree_build_interrupted);
 		}
 	interrupt->label("Cancel");
 	distance->clear();
@@ -247,24 +252,20 @@ void distance_method_dialog(SEA_VIEW *view)
 	distance->value( FL_min(distance_choice, maxchoice) );
 	distance->callback(distance_callback, ignore_all_gaps);
 	treechoice->clear();
-	const Fl_Menu_Item *items = ((Fl_Menu_ *)view->menu_trees)->menu();
-	int count = ((Fl_Menu_ *)view->menu_trees)->size() - 1;
-	for(int i = view->offset_to_local_trees; i < count; i++) 
-		treechoice->add( (items + i)->label() );
+	vlength_menu *vmenu = view->menu_trees;
+	int count = view->menu_trees->vlength();
+	for (int i = 0; i < count; i++) 
+	  treechoice->add( vmenu->vitem(i)->label() );
 	treechoice->value(0);
 	treechoice->deactivate(); 
-	if(usertree->value()) bionj->setonly();
-	if(count <= view->offset_to_local_trees) {
-		usertree->deactivate(); 
-		}
-	else {
-		usertree->activate(); 
-		}
-	started = interrupted = 0;
+	if (usertree->value()) bionj->setonly();
+	if (count) usertree->activate(); 
+	else usertree->deactivate(); 
+	started = tree_build_interrupted = 0;
 	box->label("");
 	go->show();
 	w->show();
-	while(!started && !interrupted) {
+	while(!started && !tree_build_interrupted) {
 		Fl_Widget *o = Fl::readqueue();
 		if (!o) Fl::wait();
 		else if(o == usertree || o == nj || o == bionj || o == savetofile) {
@@ -273,25 +274,71 @@ void distance_method_dialog(SEA_VIEW *view)
 		}
 	distance_choice = distance->value();
 	int using_kaks = ((!view->protein) && (distance_choice == Ka || distance_choice == Ks) );
-	if(!interrupted) {
+	if(!tree_build_interrupted) {
 		sscanf(replicates->value(), "%d", &def_replicates);
 		interrupt->label("Interrupt");
 		if(bootstrap->value()) {
 			box->label("Count: 0");
-			go->hide();
 			}
+		go->hide();
+		Fl::flush();
 		run_distance_method(view, distance_choice, ignore_all_gaps->value(), bionj->value(), 
-							bootstrap->value(), def_replicates, &interrupted, box, distance->text(), using_kaks,
+							bootstrap->value(), def_replicates, box, distance->text(), using_kaks,
 							savetofile->value(), usertree->value() ? view->trees[treechoice->value()] : NULL);
 		}
 	w->hide();
 }
 
+struct dist_struct {
+  allseq *seqs;
+  int distkind;
+  int protein;
+  char **lwlseqs;
+  int in_bootstrap;
+  char *err_mess;
+  matrix *result;
+  bool done;
+};
+
+void prep_calc_dist_matrix(dist_struct *ds)
+{
+  ds->result = calc_dist_matrix(ds->seqs, ds->distkind, ds->protein, ds->lwlseqs, ds->in_bootstrap, &(ds->err_mess));
+  if (tree_build_interrupted && ds->result) {
+    Free_Mat(ds->result);
+    ds->result = NULL;
+    }
+  Fl::lock();
+  ds->done = true;
+  Fl::awake();
+  Fl::unlock();
+}
+
+matrix *run_calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs, int in_bootstrap, char **p_err_mess)
+{
+  dist_struct *ds = new dist_struct;
+  ds->seqs = seqs;
+  ds->distkind = distkind;
+  ds->protein = protein;
+  ds->lwlseqs =lwlseqs;
+  ds->in_bootstrap = in_bootstrap;
+  ds->err_mess = NULL;
+  ds->result = NULL;
+  ds->done = false;
+  tree_build_interrupted = false;
+  fl_create_thread( (void (*)(void*))prep_calc_dist_matrix, ds, 0);
+  while (!ds->done) Fl::wait();
+  matrix* m = ds->result;
+  if (p_err_mess) *p_err_mess = ds->err_mess;
+  delete ds;
+  return m;
+}
+
+
 void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int use_bionj, int use_bootstrap,
-	int replicates, int *interrupted, Fl_Box *box, const char *distance_name, int using_kaks, int savetofile,
+	int replicates, Fl_Box *box, const char *distance_name, int using_kaks, int savetofile,
 	const char *user_tree)
 {
-	char *display_tree, tree_label[100]="";
+	char *display_tree, *err_mess, tree_label[100]="";
 	int total = 0, i, j;
 	char **lwlseqs = NULL;
 	if(using_kaks) remove_all_gaps = FALSE;//keep gaps not to alter reading frame
@@ -312,11 +359,17 @@ void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 		lwlseqs = (char **)malloc(phyml_seqs->n_otu * sizeof(char *));
 		for(i = 0; i < phyml_seqs->n_otu; i++) lwlseqs[i] = phyml_seqs->c_seq[i]->state;
 		}
-	matrix *phyml_mat = calc_dist_matrix(phyml_seqs, distkind, view->protein, lwlseqs, FALSE);
+	matrix *phyml_mat = run_calc_dist_matrix(phyml_seqs, distkind, view->protein, lwlseqs, FALSE, &err_mess);
 	if(phyml_mat == NULL) {
 	  free_after_view_to_allseq(phyml_seqs);
-	  if( using_kaks ) free(lwlseqs);
-	  else fl_alert("Can't compute distances because sequences are too divergent.\nAre they aligned?");
+	  if( using_kaks ) {
+	    free(lwlseqs);
+	    if (err_mess) {
+	      fl_alert("%s", err_mess);
+	      delete[] err_mess;
+	      }
+	    }
+	  else if (!tree_build_interrupted) fl_alert("Can't compute distances because sequences are too divergent.\nAre they aligned?");
 	  return;
 	  }
 	if(user_tree != NULL) {//process user tree
@@ -366,11 +419,11 @@ void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 		free(full_tree);
 		if(!error) {
 			for(int r = 0; r < replicates; r++) {
-				if(*interrupted) break;
+				if (tree_build_interrupted) break;
 				bootstrap_weights(phyml_seqs, using_kaks );
-				phyml_mat = calc_dist_matrix(phyml_seqs, distkind, view->protein, lwlseqs, TRUE);
+				phyml_mat = run_calc_dist_matrix(phyml_seqs, distkind, view->protein, lwlseqs, TRUE, &err_mess);
 				if(phyml_mat == NULL) {
-					total = 0;
+					if (!tree_build_interrupted) total = 0;
 					break;
 					}
 				phyml_mat->method = ! use_bionj; //1:NJ 0:BioNJ
@@ -422,6 +475,7 @@ void savedistinfile(matrix *mat, const char *dirname)
 	char fname[300] = "";
 	Fl_Native_File_Chooser *chooser = new Fl_Native_File_Chooser();
 	chooser->type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);
+  chooser->options(Fl_Native_File_Chooser::SAVEAS_CONFIRM);
 	chooser->title("Set distance output filename");  
 #ifdef WIN32
 	char separ = '\\';
@@ -450,7 +504,7 @@ void savedistinfile(matrix *mat, const char *dirname)
 	fclose(out);
 }
 
-matrix *calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs, int in_bootstrap)
+matrix *calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs, int in_bootstrap, char **p_err_mess)
 {
 	matrix *phyml_mat;
 	if(distkind == observed_pdist || distkind == observed_ndist) {//observed
@@ -487,6 +541,7 @@ matrix *calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs
 	else if( (distkind == Ka || distkind == Ks) && ! protein) {//Ka or Ks
 		int sat1, sat2, count = 0, err = 0, i, j;
 		int l3 = 3*(seqs->clean_len / 3);//dismiss partial last codon
+		*p_err_mess = NULL;
 		static int first = TRUE;
 		if(first) {
 			first = FALSE;
@@ -504,12 +559,16 @@ matrix *calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs
 			else //Ks
 				err = fastlwl(lwlseqs, seqs->n_otu, l3, NULL, phyml_mat->dist, NULL, NULL, &sat1, &sat2, seqs->wght);
 			}
-		while(in_bootstrap && err && count <=50);
-		if(err) {
-			fl_alert("Saturation between %s and %s", 
-									   phyml_mat->name[sat1], phyml_mat->name[sat2]);
-			return NULL;
-			}
+		while (in_bootstrap && err && count <=50);
+		if (err) {
+		    if (!in_bootstrap && !tree_build_interrupted) {
+			char *err_mess = new char[200];
+			snprintf(err_mess, 200, "Saturation between %s and %s", phyml_mat->name[sat1], phyml_mat->name[sat2]);
+			*p_err_mess = err_mess;
+		    }
+		    Free_Mat(phyml_mat);
+		    return NULL;
+		    }
 		for(i = 0; i < seqs->n_otu - 1; i++) {
 			for(j = i + 1; j < seqs->n_otu; j++) {
 				phyml_mat->dist[j][i] = phyml_mat->dist[i][j];
@@ -520,17 +579,17 @@ matrix *calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs
 	return phyml_mat;
 }
 
-
 Fl_Input *name;
 Fl_Choice *mymodel;
 Fl_Round_Button *b_alrt; Fl_Round_Button *b_no; Fl_Int_Input *b_count;
+Fl_Round_Button *f_empir;
 Fl_Round_Button *r_estim; Fl_Float_Input *r_val;
 Fl_Round_Button *i_no; Fl_Round_Button *i_estim; Fl_Float_Input *i_val;
 Fl_Round_Button *g_no; Fl_Choice *g_cats; Fl_Round_Button *a_est; Fl_Float_Input *a_val;
 Fl_Round_Button *nni_b, *spr_b, *nni_spr_b;
 Fl_Round_Button *u_bionj, *u_menutree, *u_random;
 Fl_Choice *u_choice;
-Fl_Check_Button *u_optimize;
+Fl_Check_Button *u_optimize, *u_quiet;
 Fl_Int_Input *u_random_count;
 char *phyml_path;
 void phyml_dialog(SEA_VIEW *view)
@@ -541,14 +600,22 @@ void phyml_dialog(SEA_VIEW *view)
 		char *p, *q;
 		p = get_res_value("phyml",
 #ifdef WIN32
-					"phyml_3.0.1_win32.exe"
+					"PhyML_3.0_win32.exe"
 #elif defined(__APPLE__)
-						  "phyml_3.0_universal"
+#ifdef __ppc__
+				  "phyml_3.0_universal"
+#else
+				  "PhyML_3.0_macOS_i386"
+#endif
 #elif defined(sparc)
 						  "phyml_3.0_sparc"
 #else
 #ifndef PHYMLNAME
-#define PHYMLNAME "phyml_3.0.1_linux32"
+#ifdef __i386__
+#define PHYMLNAME "PhyML_3.0_linux32"
+#else
+#define PHYMLNAME "PhyML_3.0_linux64"
+#endif
 #endif
 					PHYMLNAME
 #endif
@@ -589,6 +656,7 @@ void phyml_dialog(SEA_VIEW *view)
 		mymodel->add("MtArt");
 		mymodel->add("HIVw");
 		mymodel->add("HIVb");
+	  mymodel->add("FLU");
 		mymodel->value(0);
 		}
 	else {
@@ -620,6 +688,20 @@ void phyml_dialog(SEA_VIEW *view)
 	b_count->static_value("100");
 	b_group->end();
 	y += b_count->h() + 30;
+  // freq + Ts/Tv group
+  Fl_Group *fr_group = new Fl_Group(0, y, w->w(), 75, "");
+  // frequencies group
+  Fl_Group *f_group = new Fl_Group(0, y, w->w(), 30, view->protein?"Amino acid equilibrium frequencies":"Nucleotide equilibrium frequencies");
+  f_group->box(FL_ROUNDED_BOX);
+  f_group->align(FL_ALIGN_TOP|FL_ALIGN_CENTER);
+  y += 5;
+  f_empir = new Fl_Round_Button(3, y, 85, 20, "Empirical");
+  f_empir->type(FL_RADIO_BUTTON);
+  Fl_Round_Button *f_m = new Fl_Round_Button(f_empir->x() + f_empir->w() + 5, y, 60, 20, view->protein?"Model-given":"Optimized");
+  f_m->type(FL_RADIO_BUTTON);
+  (view->protein?f_m:f_empir)->setonly();
+  f_group->end();
+  y += f_group->h() + 15;
 	//Ts/Tv ratio
 	Fl_Group *r_group = new Fl_Group(0, y, w->w(), 30, "Ts/Tv ratio");
 	r_group->box(FL_ROUNDED_BOX);
@@ -633,9 +715,11 @@ void phyml_dialog(SEA_VIEW *view)
 	r_val = new Fl_Float_Input(r_fix->x() + r_fix->w() + 10, y, 40, 20, "");
 	r_val->value("4.0");
 	r_group->end();
-	if(!view->protein) mymodel->callback(model_callback, r_group);
 	r_group->deactivate();
-	y += r_group->h() + 15;
+	if (view->protein) r_group->hide();
+	else y += r_group->h() + 15;
+  fr_group->end();
+  if (!view->protein) mymodel->callback(model_callback, fr_group);
 	//invariable
 	Fl_Group *i_group = new Fl_Group(0, y, w->w(), 30, "Invariable sites");
 	i_group->box(FL_ROUNDED_BOX);
@@ -709,11 +793,11 @@ void phyml_dialog(SEA_VIEW *view)
 	u_menutree = new Fl_Round_Button(3, y, 95, 20, "User given:");
 	u_menutree->type(FL_RADIO_BUTTON);
 	u_choice = new Fl_Choice(u_menutree->x() + u_menutree->w() + 5, y, 100, 20, "");
-	const Fl_Menu_Item *items = ((Fl_Menu_ *)view->menu_trees)->menu();
-	int count = ((Fl_Menu_ *)view->menu_trees)->size() - 1;
-	for(int i = view->offset_to_local_trees; i < count; i++) u_choice->add( (items + i)->label() );
+	vlength_menu *vmenu = view->menu_trees;
+	int count = view->menu_trees->vlength();
+	for(int i = 0; i < count; i++) u_choice->add( vmenu->vitem(i)->label() );
 	u_choice->value(0);
-	if(count <= view->offset_to_local_trees) {u_menutree->deactivate(); u_choice->deactivate(); }
+	if(count <= 0) {u_menutree->deactivate(); u_choice->deactivate(); }
 	u_random = new Fl_Round_Button(3, u_menutree->y() + u_menutree->h() + 5, 80, 20, "Add: ");
 	u_random->deactivate();
 	u_random->type(FL_RADIO_BUTTON);
@@ -727,6 +811,11 @@ void phyml_dialog(SEA_VIEW *view)
 	cancel->callback(cancel_callback, NULL);
 	Fl_Return_Button *ok = new Fl_Return_Button(w->w() - 60, y, 55, 20, "Run");
 	ok->callback(run_phyml_callback, view);
+  u_quiet = new Fl_Check_Button(cancel->x() + cancel->w() + 40, y, 65, 20, "Quiet");
+  u_quiet->value(0);
+#ifndef WIN32
+  //u_quiet->hide();
+#endif
 	w->end();
 	w->size(w->w(), y + cancel->h() + 5);
 	w->callback(cancel_callback, NULL);
@@ -749,11 +838,14 @@ static void cancel_callback(Fl_Widget *ob, void *data)
 
 static void model_callback(Fl_Widget *ob, void *data)
 {
-	Fl_Choice *c = (Fl_Choice *)ob;
-	Fl_Group *g = (Fl_Group *)data;
-	int v = c->value();
-	if(v == 1 || v == 3 || v == 4 || v == 5) g->activate();
-	else g->deactivate();
+  Fl_Choice *c = (Fl_Choice *)ob;
+  Fl_Group *fr_group = (Fl_Group*)data;
+  Fl_Group *f_group = (Fl_Group *)fr_group->child(0);
+  Fl_Group *r_group = (Fl_Group *)fr_group->child(1);
+  int v = c->value();
+  if (v >=0 && v <= 2) f_group->deactivate(); else f_group->activate();
+  if (v == 1 || v == 3 || v == 4 || v == 5) r_group->activate();
+  else r_group->deactivate();
 }
 
 
@@ -853,9 +945,19 @@ void run_phyml_callback(Fl_Widget *ob, void *data)
 	  sprintf(args + strlen(args), " -t %s", r_val->value());
 	  }
   }
+  if( (view->protein) || 
+     (strcmp(modelname,"JC69")!=0 && strcmp(modelname,"K80")!=0 && strcmp(modelname,"F81")!=0) ) {
+    if(f_empir->value()) {
+      strcat(args, " -f e");
+    }
+    else {
+      strcat(args, " -f m");
+    }
+  }
   if(spr_b->value()) strcat(args, " -s SPR");
   else if(nni_spr_b->value()) strcat(args, " -s BEST");
   if(!u_optimize->value()) { strcat(args, " -o lr"); user_tree = TRUE; }
+  if (u_quiet->value()) strcat(args, " --quiet");
   if(u_menutree->value()) {
 	   char *tree = view->trees[u_choice->value()];
 	   if(*tree == '[' && (p = strchr(tree, ']')) != NULL) { //remove tree header
@@ -1126,6 +1228,7 @@ matrix *Obs_Dist(allseq *data, model *mod)
 	
 	Fors(site,data->c_seq[0]->len,mod->stepsize)
     {
+      if (tree_build_interrupted) break;
 		For(j,data->n_otu-1)
 		{
 			for(k=j+1;k<data->n_otu;k++)
@@ -1180,6 +1283,7 @@ matrix *Kimura_p_Dist(allseq *data)
 	jcmodel.ns = 20;
 	matrix *mat = Obs_Dist(data, &jcmodel);
 	for(i = 0; i < data->n_otu; i++) {
+	  if (tree_build_interrupted) break;
 		for(j = i + 1; j < data->n_otu; j++) {
 		  phydbl x = 1 - mat->dist[i][j] - 0.2 * mat->dist[i][j] * mat->dist[i][j];
 		  if(x <= 0) {
@@ -1267,6 +1371,7 @@ matrix *K80_dist_nogamma(allseq *data)
 	
 	For(i,data->c_seq[0]->len)
     {
+      if (tree_build_interrupted) break;
 		For(j,data->n_otu-1)
 		{
 			for(k=j+1;k<data->n_otu;k++)
@@ -1361,6 +1466,7 @@ matrix *HKY_dist(allseq *data)
   {
 	  for(j = i + 1; j < data->n_otu; j++)
 	  {
+	    if (tree_build_interrupted) return mat;
 		x = hky(data->c_seq[i]->state, data->c_seq[j]->state, data->clean_len, data->wght);
 		if(x == -1) {
 		  Free_Mat(mat);
@@ -1547,10 +1653,12 @@ void jumble_fct(char** seq, char** name, int notu)
 
 void parsimony_dialog(SEA_VIEW *view)
 {
-	int steps, info, njumbles, i, count, maxbest, started = 0, interrupted = 0, use_bootstrap, 
+	int steps, info, njumbles, i, count, maxbest, started = 0, use_bootstrap, 
 		nreplicates, replicates_done, user_tree, nogaps; 
-	char *tree, *final_tree;
+	char *tree = NULL, *final_tree;
 	const char *p, *progname;
+  
+	tree_build_interrupted = 0;
 	Fl_Window *w = new Fl_Window(255, 225);
 	w->set_modal();
 	w->label("Parsimony analysis");
@@ -1580,31 +1688,32 @@ void parsimony_dialog(SEA_VIEW *view)
 	Fl_Round_Button *w_user = new Fl_Round_Button(2, w_count->y() + w_count->h() + 5, 85, 20, "User tree:");
 	w_user->type(FL_RADIO_BUTTON);
 	Fl_Choice *w_choice = new Fl_Choice(w_user->x() + w_user->w() + 5, w_user->y(), 130, 20, "");
-	const Fl_Menu_Item *items = ((Fl_Menu_ *)view->menu_trees)->menu();
-	count = ((Fl_Menu_ *)view->menu_trees)->size() - 1;
-	for( i = view->offset_to_local_trees; i < count; i++) w_choice->add( (items + i)->label() );
+	count = view->menu_trees->vlength();
+	for( i = 0; i < count; i++) w_choice->add( view->menu_trees->vitem(i)->label() );
 	w_choice->value(0);
-	if(count <= view->offset_to_local_trees) {w_user->deactivate(); w_choice->deactivate(); }
+	if (!count) {w_user->deactivate(); w_choice->deactivate(); }
 	
 	Fl_Button *w_interrupt = new Fl_Button(w_dobootstrap->x(), w_user->y() + w_user->h() + 5, 60, 20, "Cancel");
 	Fl_Return_Button *w_ok = new Fl_Return_Button(w->w() - 65, w_interrupt->y(), 60, 20, "OK");
-	w_interrupt->callback(interrupt_callback, &interrupted);
+	w_interrupt->callback(interrupt_callback, &tree_build_interrupted);
 	w_ok->callback(interrupt_callback, &started);
 	w->end();
-	w->callback(interrupt_callback, &interrupted);
+	w->callback(interrupt_callback, &tree_build_interrupted);
 	w->size(w->w(), w_ok->y() + w_ok->h() + 5);
 	w->show();
-	while(!started && !interrupted) {
+	while(!started && !tree_build_interrupted) {
 	  Fl::wait();
 	  if(w_gapsunknown->value()) { w_nogaps->value(0); w_nogaps->deactivate(); }
 	  else w_nogaps->activate();
 	  if(w_nogaps->value()) { w_gapsunknown->value(0); w_gapsunknown->deactivate(); }
 	  else w_gapsunknown->activate();
 	  }
-	if(interrupted) {
+	if (tree_build_interrupted) {
 		delete w;
 		return;
 	}
+	w_ok->hide();
+	w_interrupt->label("Interrupt");
 	p = w_best->value();
 	sscanf(p, "%d", &maxbest);
 	if(w_dojumble->value()) {
@@ -1623,15 +1732,12 @@ void parsimony_dialog(SEA_VIEW *view)
 	if(use_bootstrap) {
 		p = w_btrepl->value();
 		sscanf(p, "%d", &nreplicates);
-		w_interrupt->label("Interrupt");
-		w_ok->hide();
 		}
 	else {
 		if(user_tree) user_tree = w_choice->value() + 1;
-		else delete w; 
 	}
 
-	my_watch_cursor(view->dnawin);
+	my_watch_cursor(w);
 	allseq *tmpallseqs = view_to_allseq(view, nogaps);
 	if(w_gapsunknown->value()) gaps_as_unknown_states(tmpallseqs, view->protein);
 	char **pars_seqs = (char **)malloc(tmpallseqs->n_otu * sizeof(char *));
@@ -1641,14 +1747,22 @@ void parsimony_dialog(SEA_VIEW *view)
 		pars_names[i] = tmpallseqs->c_seq[i]->name;
 	}
 	info = informative(pars_seqs, tmpallseqs->n_otu);
-	if(view->protein) progname = "Protpars"; else progname = "Dnapars";
+	if(view->protein) progname = "Protpars"; 
+	else {
+	  progname = "Dnapars";
+	  if (tmpallseqs->clean_len >= SHRT_MAX) {
+	    fl_alert("Can't compute parsimony with DNA seqs longer than %d.", SHRT_MAX);
+	    goto way_out;
+	    }
+	  }
 	if(use_bootstrap) {
 		tree = parsimony_with_bootstrap(tmpallseqs, pars_seqs, pars_names, tmpallseqs->n_otu, njumbles, 
-				view->protein, &steps, maxbest, nreplicates, w_count, &interrupted, &replicates_done);
-		delete w;
-		final_tree = (char *)malloc(strlen(tree) + 150);
-		sprintf(final_tree, "[%s, bootstrap with %d replic., %d steps, %d sites (%d informative)%s]%s",
+				view->protein, &steps, maxbest, nreplicates, w_count, &replicates_done);
+		if (tree) {
+		  final_tree = (char *)malloc(strlen(tree) + 150);
+		  sprintf(final_tree, "[%s, bootstrap with %d replic., %d steps, %d sites (%d informative)%s]%s",
 				progname, replicates_done, steps, tmpallseqs->clean_len, info, gapsunknown, tree);
+		  }
 		}
 	else if(user_tree) {
 		char *differs, *input_tree;
@@ -1658,59 +1772,67 @@ void parsimony_dialog(SEA_VIEW *view)
 		while(*p == ' ') p++;
 		input_tree = strdup(p);
 		i = compare_newick_with_names(input_tree, pars_names, tmpallseqs->n_otu, &differs);
-		if(i != 0) {
+		if (i != 0) {
 			fl_alert("Names in user-tree and target sequences differ:\n%s in %s only", 
 					 differs, i == 1 ? "user-tree" : "target sequences");
-			delete w;
-			fl_reset_cursor(view->dnawin);
-			free_after_view_to_allseq(tmpallseqs);
-			free(pars_seqs);
-			free(pars_names);
-			return;
-			}
-		delete w;
-		input_tree = (char *)realloc(input_tree, strlen(input_tree) + 4 * tmpallseqs->n_otu + 6 );
-		if( make_binary_or_unrooted(input_tree) != NULL) return;
-		strcat(input_tree, ";");
-		if(view->protein) tree = protpars(pars_seqs, pars_names, tmpallseqs->n_otu, &steps, input_tree, maxbest, NULL);
-		else tree = dnapars(pars_seqs, pars_names, tmpallseqs->n_otu, &steps, input_tree, maxbest, NULL);
-		free(input_tree);
-		if(tree == NULL) {
-			fl_alert("Error in user-tree");
+			tree = NULL;
 			}
 		else {
-			final_tree = (char *)malloc(strlen(tree) + 150);
-			sprintf(final_tree, "[%s, user-tree, %d steps, %d sites (%d informative)%s]%s",
-					progname, steps, tmpallseqs->clean_len, info, gapsunknown, tree);
-			}
+		  input_tree = (char *)realloc(input_tree, strlen(input_tree) + 4 * tmpallseqs->n_otu + 6 );
+		  if( make_binary_or_unrooted(input_tree) != NULL) return;
+		  strcat(input_tree, ";");
+		  tree = parsimony(pars_seqs, pars_names, tmpallseqs->n_otu, &steps, input_tree, maxbest, NULL, view->protein);
+		  free(input_tree);
+		  if(tree == NULL) {
+			  fl_alert("Error in user-tree");
+			  }
+		  else {
+			  final_tree = (char *)malloc(strlen(tree) + 150);
+			  sprintf(final_tree, "[%s, user-tree, %d steps, %d sites (%d informative)%s]%s",
+					  progname, steps, tmpallseqs->clean_len, info, gapsunknown, tree);
+			  }
+		}
 	}
 	else {
-		tree = parsimony_with_jumble(pars_seqs, pars_names, tmpallseqs->n_otu, njumbles, view->protein, 
-								 &steps, &count, maxbest);
+	  tree = parsimony_with_jumble(pars_seqs, pars_names, tmpallseqs->n_otu, njumbles, view->protein, 
+								 &steps, &count, maxbest, w_count);
+	  if (tree) {
 		final_tree = (char *)malloc(strlen(tree) + 150);
 		sprintf(final_tree, "[%s, %d best trees%s, %d steps, %d sites (%d informative)%s]%s",
 			progname, count, count > 1 ? " (strict consensus)" : "", steps, tmpallseqs->clean_len, info, gapsunknown, tree);
 		}
-	if(tree != NULL) free(tree);
+	  }
+way_out:
+	fl_reset_cursor(w);
+	delete w;
 	free_after_view_to_allseq(tmpallseqs);
 	free(pars_seqs);
 	free(pars_names);
-	fl_reset_cursor(view->dnawin);
-	if(tree != NULL) treedraw(final_tree, view, progname, FALSE);
+	if (tree != NULL) {
+	  free(tree);
+	  treedraw(final_tree, view, progname, FALSE);
+	  }
 }
 
 
 char *parsimony_with_jumble(char **pars_seqs, char **pars_names, int n_otu, int njumbles, int protein, int *psteps,
-	int *pcount, int maxbest)
+	int *pcount, int maxbest, Fl_Widget *w_count)
 {
 	int i, steps, newsteps, count = 0;
 	char *tree, *newtree, *p, *q;
-	if(protein) tree = protpars(pars_seqs, pars_names, n_otu, &steps, NULL, maxbest, NULL);
-	else tree = dnapars(pars_seqs, pars_names, n_otu, &steps, NULL, maxbest, NULL);
+	if (njumbles > 0) {
+	  w_count->label("Initial sequence order");
+	  }
+	tree = parsimony(pars_seqs, pars_names, n_otu, &steps, NULL, maxbest, NULL, protein);
+	if (!tree) return NULL;
+	newsteps = steps;
 	for( i = 1; i <= njumbles; i++) {
+		char tmp[40];
+		sprintf(tmp, "Jumble attempt: %d steps: %d", i, newsteps); 
+		w_count->label(tmp); 
 		jumble_fct(pars_seqs, pars_names, n_otu);
-		if(protein) newtree = protpars(pars_seqs, pars_names, n_otu, &newsteps, NULL, maxbest, NULL);
-		else newtree = dnapars(pars_seqs, pars_names, n_otu, &newsteps, NULL, maxbest, NULL);
+		newtree = parsimony(pars_seqs, pars_names, n_otu, &newsteps, NULL, maxbest, NULL, protein);
+		if (!newtree) break;
 		if(newsteps < steps) {
 			steps = newsteps;
 			free(tree);
@@ -1747,33 +1869,29 @@ char *parsimony_with_jumble(char **pars_seqs, char **pars_names, int n_otu, int
 
 
 char *parsimony_with_bootstrap(allseq *tmpallseqs, char **pars_seqs, char **pars_names, int n_otu, int njumbles, int protein, int *psteps,
-			int maxbest, int replicates, Fl_Box *w_count, int *pinterrupted, int *preplicates_done)
+			int maxbest, int replicates, Fl_Box *w_count, int *preplicates_done)
 {
 	int i, j, steps, newsteps, count = 0, replicates_done = 0;
 	char *tree, *newtree, *p;
 	
+	w_count->label("Full sequence tree"); 
 	tree = parsimony_with_jumble(pars_seqs, pars_names, n_otu, njumbles, protein, 
-								 &steps, &count, maxbest);
+								 &steps, &count, maxbest, w_count);
+	if (!tree) return NULL;
 	*psteps = steps;
 	init_add_bootstrap(tree);
 	for(j = 0; j < replicates; j++) {
 		char tmp[30];
 		sprintf(tmp, "Bootstrap replicate: %d", j + 1); 
 		w_count->label(tmp); 
-		w_count->redraw(); 
-#ifdef WIN32
-		Fl::check();
-#else
-		while( Fl::wait(0) != 0) ;
-#endif
-		if(*pinterrupted) break;
+		if (tree_build_interrupted) break;
 		bootstrap_weights(tmpallseqs, FALSE);
-		if(protein) tree = protpars(pars_seqs, pars_names, n_otu, &steps, NULL, maxbest, tmpallseqs->wght);
-		else tree = dnapars(pars_seqs, pars_names, n_otu, &steps, NULL, maxbest, tmpallseqs->wght);
+		tree = parsimony(pars_seqs, pars_names, n_otu, &steps, NULL, maxbest, tmpallseqs->wght, protein);
+		if (!tree) break;
 		for( i = 1; i <= njumbles; i++) {
 			jumble_fct(pars_seqs, pars_names, n_otu);
-			if(protein) newtree = protpars(pars_seqs, pars_names, n_otu, &newsteps, NULL, maxbest, tmpallseqs->wght);
-			else newtree = dnapars(pars_seqs, pars_names, n_otu, &newsteps, NULL, maxbest, tmpallseqs->wght);
+			newtree = parsimony(pars_seqs, pars_names, n_otu, &newsteps, NULL, maxbest, tmpallseqs->wght, protein);
+			if (!newtree) break;
 			if(newsteps < steps) {
 				steps = newsteps;
 				free(tree);
@@ -1804,6 +1922,56 @@ char *parsimony_with_bootstrap(allseq *tmpallseqs, char **pars_seqs, char **pars
 	return tree;
 }
 
+struct pars_struct {
+  char** seq;
+  char** seqname;
+  int notu;
+  int *steps;
+  char* toevaluate;
+  int arg_maxtrees;
+  int *bt_weights;
+  char *result;
+  bool protein;
+  bool done;
+};
+
+static void parsimony_thread(pars_struct* args)
+{
+  typedef char* (*pars_f)(char** seq, char** seqname, int notu, int *steps, char* toevaluate, int arg_maxtrees, int *bt_weights);
+  pars_f pf = args->protein ? protpars : dnapars;
+  args->result = (*pf)(args->seq, args->seqname, args->notu, args->steps, args->toevaluate, args->arg_maxtrees, args->bt_weights);
+  Fl::lock();
+  args->done = true;
+  Fl::awake();
+  Fl::unlock();
+}
+
+char *parsimony(char** seq, char** seqname, int notu, int *steps, char* toevaluate, int arg_maxtrees, int *bt_weights, bool protein)
+{
+  pars_struct *args = (pars_struct*)malloc(sizeof(pars_struct));
+  args->seq = seq;
+  args->seqname = seqname;
+  args->notu = notu;
+  args->steps = steps;
+  args->toevaluate = toevaluate;
+  args->arg_maxtrees = arg_maxtrees;
+  args->bt_weights = bt_weights;
+  args->result = NULL;
+  args->protein = protein;
+  args->done = false;
+  tree_build_interrupted = 0;
+  if ( ! fl_create_thread((void (*) (void *))parsimony_thread, args, 1024) ) {
+    while (!args->done) Fl::wait();
+  }
+  char *p = args->result;
+  free(args);
+  if (tree_build_interrupted && p) {
+    free(p);
+    p = NULL;
+  }
+  return p;
+}
+
 
 /* from Numerical Recipes in C */
 /* Allocation function for ludcmp function */
@@ -1945,6 +2113,7 @@ matrix *LOGDET_dist(allseq *data)
     {
 		for(j = i + 1; j < data->n_otu; j++)
 		{
+		  if (tree_build_interrupted) return mat;
 		  double x = logdet(data->c_seq[i]->state, data->c_seq[j]->state, data->clean_len, data->wght);
 		  if(x == -1.) {
 			Free_Mat(mat);
@@ -2031,4 +2200,3 @@ char *replace_tree_names_by_rank(const char *tree, char **names, int notu)
 	*q = 0;
 	return newtree;
 }
-
