@@ -2,10 +2,9 @@
 #include <time.h>
 #include "unrooted.h"
 #include "treedraw.h"
-#ifdef HAS_PRINTER
+#include "pdf_or_ps.h"
 #include <FL/Fl_Printer.H>
 #include <FL/Fl_PostScript.H>
-#endif
 
 
 /* prototypes of included functions */
@@ -54,9 +53,7 @@ double length_log_phys(FD_unrooted *data, double p);
 double length_phys_log(FD_unrooted *data, double p);
 void squared2circular(FD_nj_plot *fd_nj_plot, Fl_Choice *o);
 void circular2squared(FD_nj_plot *fd_nj_plot);
-#ifdef HAS_PRINTER
-void print_unrooted(FD_unrooted *fd_unrooted, bool to_ps_file);
-#endif
+void print_unrooted(FD_unrooted *fd_unrooted, bool to_ps_file, const char *directname = NULL);
 
 
 /* external functions */
@@ -73,7 +70,6 @@ extern char *nextpar(char *pospar);
 extern double arrondi_echelle(double x);
 extern void zoom_callback(Fl_Widget *wgt, void *data);
 extern void scroller_callback(Fl_Widget *wgt, void *data);
-extern void draw_rotated_string(const char *txt, int x, int y, float degrees /*clockwise*/);
 extern int calc_brl_for_lengthless(struct noeud *centre, struct noeud *pere);
 extern const char *preptree(FD_nj_plot *fd_nj_plot);
 extern void free_tree(FD_nj_plot *fd_nj_plot);
@@ -82,7 +78,7 @@ extern char *ecrit_arbre_parenth(FD_nj_plot *fd_nj_plot, struct noeud *root);
 
 /* globals */
 static int nextotu, num_noeud;
-extern paperformat printout_pageformat;
+extern Fl_Paged_Device::Page_Format printout_pageformat;
 
 
 void rooted_unrooted_callback(Fl_Widget *o, void *data)
@@ -153,8 +149,8 @@ void rooted_unrooted_callback(Fl_Widget *o, void *data)
 
 void squared2circular(FD_nj_plot *fd_nj_plot, Fl_Choice *o)
 {
-	Fl_Menu_Item *item = (Fl_Menu_Item *)fd_nj_plot->menu_file->menu();
-	Fl_Menu_Item *edit_item = (Fl_Menu_Item *)fd_nj_plot->menu_edit->menu();
+	Fl_Menu_Item *item = fd_nj_plot->menu_file->get_menu();
+  	Fl_Menu_Item *edit_item = fd_nj_plot->menu_edit->get_menu();
 	Fl_Widget *panel = fd_nj_plot->panel;
 	FD_unrooted *fd_unrooted;
 	char *tree = ecrit_arbre_parenth_unrooted(fd_nj_plot, fd_nj_plot->racine);
@@ -218,8 +214,8 @@ void squared2circular(FD_nj_plot *fd_nj_plot, Fl_Choice *o)
 
 void circular2squared(FD_nj_plot *fd_nj_plot)
 {
-  Fl_Menu_Item *item = (Fl_Menu_Item *)fd_nj_plot->menu_file->menu();
-  Fl_Menu_Item *edit_item = (Fl_Menu_Item *)fd_nj_plot->menu_edit->menu();
+  Fl_Menu_Item *item = fd_nj_plot->menu_file->get_menu();
+  Fl_Menu_Item *edit_item = fd_nj_plot->menu_edit->get_menu();
   FD_unrooted *fd_unrooted = (FD_unrooted *)fd_nj_plot->fd_unrooted;
   fd_unrooted->unrooted_plot->hide();
   fd_unrooted->vert_scroller->hide();
@@ -425,23 +421,19 @@ void draw_tree(FD_unrooted *ob)
 }
 
 
-#ifdef HAS_PRINTER
-void print_unrooted(FD_unrooted *fd_unrooted, const char *name, bool to_ps_file)
+void print_unrooted(FD_unrooted *fd_unrooted, const char *name, bool to_ps_file, const char *directname)
 {
   Fl_Printer* myprinter;
-  int old_x, old_y;
+  int old_x, old_y, error;
   old_x = fd_unrooted->x_offset;
   old_y = fd_unrooted->y_offset;
   if (to_ps_file) {
-#ifdef NO_PDF
-    myprinter = (Fl_Printer*)new Fl_PostScript_File_Device();
-    if (((Fl_PostScript_File_Device*)myprinter)->start_job (1, 
-	      printout_pageformat == A4 ? Fl_Paged_Device::A4 : Fl_Paged_Device::LETTER)) return;
-#else
-    myprinter = (Fl_Printer*)new Fl_PDF_File_Device();
-    if (((Fl_PDF_File_Device*)myprinter)->start_job(name, printout_pageformat == A4 ? 
-						    Fl_Paged_Device::A4 : Fl_Paged_Device::LETTER)) return;
-#endif
+    myprinter = (Fl_Printer*)new Fl_PDF_or_PS_File_Device();
+    if (directname) 
+      error = ((Fl_PDF_or_PS_File_Device*)myprinter)->begin_document(directname, printout_pageformat);
+    else
+      error = ((Fl_PDF_or_PS_File_Device*)myprinter)->start_job(name, printout_pageformat) ;
+    if (error) return;
     }
   else {
     int from, to;
@@ -449,9 +441,9 @@ void print_unrooted(FD_unrooted *fd_unrooted, const char *name, bool to_ps_file)
     if (myprinter->start_job(1, &from, &to) ) return;
   }
 #ifndef NO_PDF
-  PDF *pdf;
-  if (to_ps_file) pdf = ((Fl_PDF_File_Device*)myprinter)->pdf();
-  if ( (!to_ps_file) || setjmp(pdf_jbuf(pdf)->jbuf) == 0) { // replaces PDF_TRY
+  jmp_buf *jbuf;
+  if (to_ps_file) ((Fl_PDF_File_Device*)myprinter)->surface_try(&jbuf);
+  if ( (!to_ps_file) || setjmp(*jbuf) == 0) { // replaces PDF_TRY
 #endif    
     myprinter->start_page();
     fd_unrooted->previous_w = fd_unrooted->previous_h = 0;
@@ -476,7 +468,7 @@ void print_unrooted(FD_unrooted *fd_unrooted, const char *name, bool to_ps_file)
     myprinter->end_job();
 #ifndef NO_PDF
     } // end of PDF_TRY
-    if (to_ps_file && pdf_catch(pdf)) { // replaces PDF_CATCH
+    if (to_ps_file && ((Fl_PDF_File_Device*)myprinter)->surface_catch()) { // replaces PDF_CATCH
       ((Fl_PDF_File_Device*)myprinter)->error_catch();
     }
 #endif
@@ -485,8 +477,6 @@ void print_unrooted(FD_unrooted *fd_unrooted, const char *name, bool to_ps_file)
   fd_unrooted->y_offset = old_y;
 }
 
-#endif
-
 
 int debut_arbre(const char *tree, FD_unrooted *fd_unrooted)
 //returns 0 iff OK
@@ -667,11 +657,7 @@ hit draw_name_angle(cp_point *phys_pos, const char *nom, FD_unrooted *fd_unroote
 	if( (!doit) && result != NO_HIT) return result;
 	if( ! doit) return NO_HIT;
 	if(color) fl_color(FL_RED);
-#if FL_MAJOR_VERSION >= 1 && FL_MINOR_VERSION >= 3
-	fl_draw(-(int)degrees, nom, x, y); // implemented in FLTK starting 1.3.x-r6784
-#else
-	draw_rotated_string(nom, x, y, -degrees);
-#endif
+	fl_draw(-(int)degrees, nom, x, y);
 	if(color) fl_color(FL_BLACK);
 	return NO_HIT;
 }
@@ -1473,92 +1459,3 @@ double calc_echelle(FD_unrooted *data, int larg)
 	return log_val;
 }
 
-
-#if USE_XFT && !defined(MICRO) && !(FL_MAJOR_VERSION >= 1 && FL_MINOR_VERSION >= 3)
-
-#include <X11/Xft/Xft.h>
-class Fl_FontSize { //taken from src/Fl_Font.H
-public:
-	Fl_FontSize *next;	// linked list for this Fl_Fontdesc
-	XftFont* font;
-	const char* encoding;
-	int size;
-	FL_EXPORT Fl_FontSize(const char* xfontname);
-	int minsize;		// smallest point size that should use this
-	int maxsize;		// largest point size that should use this
-#  if HAVE_GL
-	unsigned int listbase;// base of display list, 0 = none
-#  endif
-	FL_EXPORT ~Fl_FontSize();
-};
-struct Fl_Fontdesc {
-	const char *name;
-	char fontname[128];	// "Pretty" font name
-	Fl_FontSize *first;	// linked list of sizes of this style
-	char **xlist;		// matched X font names
-	int n;		// size of xlist, negative = don't free xlist!
-};
-extern Fl_Fontdesc* fl_fonts;
-extern Fl_FontSize* fl_fontsize;
-extern const char* fl_encoding_;
-
-static XftFont* fontopen(const char* name, float angle);
-
-void draw_rotated_string(const char *txt, int x, int y, float degrees /*clockwise*/)
-{
-	Fl_Fontdesc *font = fl_fonts + fl_font();
-	XftFont *saved = fl_fontsize->font;
-	fl_fontsize->font = fontopen(font->name, degrees );
-	fl_draw(txt, x, y);
-	XftFontClose (fl_display, fl_fontsize->font);
-	fl_fontsize->font = saved;
-}
-
-static XftFont* fontopen(const char* name, float angle) 
-{
-		XftFont *the_font; // the font we will return;
-		XftPattern *fnt_pat = XftPatternCreate(); // the pattern we will use for matching
-		int slant = XFT_SLANT_ROMAN;
-		int weight = XFT_WEIGHT_MEDIUM;
-		
-		switch (*name++) {
-			case 'I': slant = XFT_SLANT_ITALIC; break; // italic
-			case 'P': slant = XFT_SLANT_ITALIC;        // bold-italic (falls-through)
-			case 'B': weight = XFT_WEIGHT_BOLD; break; // bold
-			case ' ': break;                           // regular
-			default: name--;                           // no prefix, restore name
-		}
-		
-			XftPatternAddString(fnt_pat, XFT_FAMILY, name);
-		
-		// Construct a match pattern for the font we want...
-		XftPatternAddInteger(fnt_pat, XFT_WEIGHT, weight);
-		XftPatternAddInteger(fnt_pat, XFT_SLANT, slant);
-		XftPatternAddDouble (fnt_pat, XFT_PIXEL_SIZE, (double)fl_size_);
-		XftPatternAddString (fnt_pat, XFT_ENCODING, fl_encoding_);
-		
-		// rotate font if fl_angle_!=0
-		if (angle !=0) {
-			XftMatrix m;
-			XftMatrixInit(&m);
-			XftMatrixRotate(&m,cos(M_PI*angle/180.),sin(M_PI*angle/180.));
-			XftPatternAddMatrix (fnt_pat, XFT_MATRIX,&m);
-		}
-				
-		XftPattern *match_pat;  // the best available match on the system
-		XftResult match_result; // the result of our matching attempt
-		
-		// query the system to find a match for this font
-		match_pat = XftFontMatch(fl_display, fl_screen, fnt_pat, &match_result);
-		
-		// open the matched font
-		the_font = XftFontOpenPattern(fl_display, match_pat);
-		
-		XftPatternDestroy(fnt_pat);
-		//  XftPatternDestroy(match_pat); // FontConfig will destroy this resource for us. We must not!
-		
-		return the_font;
-}
-
-#endif
-
