@@ -1,4 +1,5 @@
 #include "seaview.h"
+#include "pdf_or_ps.h"
 #include <ctype.h>
 #include <time.h>
 #ifndef WIN32
@@ -49,13 +50,6 @@ char *save_alignment_or_region(const char *fname, char **seq, char **comments,
 	char **name_species_sets, int *sel_seqs, int tot_sel_seqs, int protein,
 	int tot_comment_lines, char **comment_name, char **comment_line, int phylipwidnames, 
 	int tot_trees, char **trees, const Fl_Menu_Item *menu_tree_items, int spaces_in_fasta_names);
-int prepare_printout(const char *filename, char **seq, int totseqs,  
-	char **seqname,
-	int *eachlength, int char_per_line,
-	int block_size, int lines_per_page, int vary_only,
-	char *align_name, int ref0);
-int calc_vary_lines(int *vary_pos, int widpos);
-void out_vary_pos(int *vary_pos, int widnames, int widpos, int nl, FILE *out);
 char *get_full_path(const char *fname);
 static void save_species_sets(int numb_species_sets, int **list_species_sets, 
 	char **name_species_sets, int totseqs, FILE *out);
@@ -64,6 +58,12 @@ void save_comment_lines(int tot_comment_lines, char **names, char **lines,
 known_format what_format(const char *filename);
 char* seaview_file_chooser_save_as(const char* message, const char* fname, SEA_VIEW *view, known_format* new_format);
 const char *extract_dirname(const char *pathname);
+int printout(SEA_VIEW *view, const char *filename, 
+	     int fontsize, int block_size, Fl_Paged_Device::Page_Format pageformat, int vary_only, int ref0, int pdfkindvalue);
+static void color_pdf_display(SEA_VIEW *view, int (*calc_color_function)( int ), char *oneline, 
+			      int widnames, double x, double y, int fontsize, double char_width, double descender,int num, int current);
+static int calc_vary_lines(int *vary_pos, int widpos);
+static void out_vary_pos(int *vary_pos, int widnames, int widpos, int nl, FILE *textfile, double x, double y);
 
 
 /* external */
@@ -238,9 +238,9 @@ void parse_trees_from_header(char *header, SEA_VIEW *view)
 			while(*p == ' ') p++;
 			q = strchr(p, '\n');
 			*q = 0;
-			int rank = ((Fl_Menu_ *)view->menu_trees)->add(p);
-			Fl_Menu_Item *items = (Fl_Menu_Item *)((Fl_Menu_ *)view->menu_trees)->menu();
-			(items + rank)->labelfont(FL_HELVETICA_ITALIC);
+		  view->menu_trees->add(p, trees_callback, NULL, 0);
+		  int rank = view->menu_trees->vlength();
+		  (view->menu_trees->vitem(rank - 1))->labelfont(FL_HELVETICA_ITALIC);
 			*q = '\n';
 			p = q + 1;
 			l = 1;
@@ -715,7 +715,7 @@ goto fini;
 int is_a_protein_seq(char *seq)
 /* returns TRUE if seq looks like a protein sequence (less than 80% ACGTU) */
 {
-static char dna[]="ACGTU";
+static char dna[]="ACGTURY";
 int total=0, length=0;
 while(*seq != 0) {
 	if(*seq != '-' && *seq != '?' && toupper(*seq) != 'N') { 
@@ -1332,206 +1332,6 @@ return err_message;
 }
 
 
-int prepare_printout(const char *filename, char **seq, int totseqs,  
-	char **seqname,
-	int *eachlength, int char_per_line,
-	int block_size, int lines_per_page, int vary_only,
-	char *align_name, int ref0)
-{
-FILE *out;
-int num, i, j, k, current, max_seq_length, fin, curr_lines, widnames, 
-	res_per_line, nl;
-int retval = TRUE;
-time_t heure;
-static char unnamed[] = "<unnamed>";
-static char num_line[200];
-int lettre;
-short *vary_need = NULL;
-int *vary_pos; /* rang ds alignement de la colonne imprime */
-
-if(totseqs == 0) return 0;
-if( ( out=fopen(filename, "w") ) == NULL) {
-	return TRUE;
-	}
-if(ref0 < 0) vary_only = FALSE;
-time(&heure);
-if(align_name == NULL) align_name = unnamed;
-fprintf(out,"Alignment:  %s\n", align_name);
-curr_lines = 1;
-if(vary_only) {
-	fprintf(out, "Displaying variable sites only.\n");
-	++curr_lines;
-	}
-fprintf(out,"Printed by seaview [blocks=%d cpl=%d lpp=%d] on %s\n",
-		block_size, char_per_line, lines_per_page, ctime(&heure));
-curr_lines += 2;
-max_seq_length = 0; widnames = 0;
-for(i=0; i<totseqs; i++) {
-	if(eachlength[i] > max_seq_length) max_seq_length = eachlength[i];
-	if( ( fin=strlen(seqname[i]) ) > widnames) widnames = fin;
-	}
-widnames += 2;
-if(vary_only) {
-	vary_need = (short *)calloc(max_seq_length, sizeof(short));
-	if(vary_need == NULL) return TRUE;
-	vary_pos = (int *)calloc(char_per_line, sizeof(int));
-	if(vary_pos == NULL) return TRUE;
-	for(i = 0; i < max_seq_length; i++) {
-		for(num = 0; num < totseqs; num++) {
-			if( toupper(seq[num][i]) != toupper(seq[ref0][i]) ) { 
-				vary_need[i] = TRUE;
-				break;
-				}
-			}
-		}
-	}
-/* nombre max de blocks qui tiennent sur une ligne de cpl chars */
-fin = (char_per_line - widnames + 1) / (block_size + 1);
-if(fin < 1) { /* garde fou */
-	fin = 1; block_size = char_per_line - widnames;
-	}
-res_per_line = fin * block_size;
-current = 0; 
-if( totseqs + 1 > lines_per_page) lines_per_page = 1000000;
-while( current < max_seq_length ) {
-	nl = 1;
-	if(vary_only) { 
-		memset(vary_pos, 0, res_per_line * sizeof(int) );
-		i = -1; j = 0; k = 0;
-		while( j < res_per_line) {
-			if(current + i >= max_seq_length) break;
-			if( !vary_need[current + ++i] ) continue;
-			j++;
-			vary_pos[k++] = current + i + 1;
-			if( j % block_size == 0) k++;
-			}
-		nl = calc_vary_lines(vary_pos,  k);
-		}
-	if(curr_lines + totseqs + nl > lines_per_page) {
-		for(i = 0; i < lines_per_page - curr_lines; i++)
-			putc('\n', out);
-		curr_lines = 0;
-		}
-	if(vary_only) {
-		out_vary_pos(vary_pos, widnames, k, nl, out);
-		curr_lines += nl;
-		}
-	else	{
-		sprintf(num_line, "%d", current + 1);
-		fin = strlen(num_line);
-		memmove(num_line + widnames - fin + 1, num_line, fin+1);
-		if(fin <= widnames) memset(num_line, ' ', widnames - fin + 1);
-		fputs(num_line, out); putc('\n', out); ++curr_lines;
-		}
-	for(num=0; num < totseqs; num++) {
-		for(j = 0; j < widnames; j++) {
-			if(seqname[num][j] == 0) break;
-			putc(seqname[num][j],out);
-			}
-		while( j < widnames) {
-			putc(' ', out); j++;
-			}
-		if(vary_only) {
-			i = -1; j = 0;
-			while( j < res_per_line) {
-				if(current + i >= max_seq_length) break;
-				if( !vary_need[current + ++i] ) continue;
-				j++;
-				if(current + i < eachlength[num]) {
-					if(num != ref0) lettre = 
-( toupper(seq[num][current+i]) == toupper(seq[ref0][current+i]) ? '.' : seq[num][current+i] );
-					else lettre = seq[ref0][current+i];
-					putc(lettre, out);
-					}
-				if( j % block_size == 0) putc(' ', out);
-				}
-			if(num == totseqs - 1) current = current + i + 1;
-			}
-
-		else	{
-			fin = res_per_line;
-			if(current+fin > eachlength[num]) 
-				fin = eachlength[num] - current;
-			if(ref0 != -1 && num != ref0) {
-				/* ecriture par reference a seq ref0 */
-				for(i=0; i<fin; i++) {
-					lettre = 
-( toupper(seq[num][current+i]) == toupper(seq[ref0][current+i]) ? '.' : seq[num][current+i] );
-					putc(lettre, out);
-					if( i < fin-1 && (i+1)%block_size == 0) 
-						putc(' ', out);
-					}
-				}
-			else	{ /* ecriture normale de seq */
-				for(i=0; i<fin; i++) {
-					putc(seq[num][current+i], out);
-					if( i < fin-1 && (i+1)%block_size == 0) 
-						putc(' ', out);
-					}
-				}
-			}
-		putc('\n', out); ++curr_lines;
-		if(ferror(out)) goto fin;
-		}
-	if(curr_lines + 1 <= lines_per_page) {
-		fprintf(out, "\n"); ++curr_lines;
-		}
-	if( ! vary_only ) current += res_per_line;
-	}
-if(ferror(out)) goto fin;
-retval = FALSE;
-fin:
-fclose(out);
-return retval;
-}
-
-
-int calc_vary_lines(int *vary_pos, int widpos)
-{
-int maxi = 0, num, nl;
-
-for(num = 0; num < widpos; num++) 
-	if(vary_pos[num] > maxi) maxi = vary_pos[num];
-if(maxi >= 100000)
-	 nl = 6;
-else if(maxi >= 10000)
-	 nl = 5;
-else if(maxi >= 1000)
-	 nl = 4;
-else if(maxi >= 100)
-	 nl = 3;
-else if(maxi >= 10)
-	 nl = 2;
-else 	
-	 nl = 1;
-return nl;
-}
-
-
-void out_vary_pos(int *vary_pos, int widnames, int widpos, int nl, FILE *out)
-{
-int num, l, echelle, digit, val;
-static char chiffre[] = "0123456789";
-
-echelle = 1;
-for(l = 2; l <= nl; l++) echelle *= 10;
-for(l = nl; l > 0; l--) {
-	for(num = 0; num < widnames; num++) putc(' ', out);
-	for(num = 0; num < widpos; num++) {
-		val = vary_pos[num];
-		if(val < echelle)
-			putc(' ', out);
-		else	{
-			digit = (val / echelle) % 10 ;
-			putc(*(chiffre + digit), out);
-			}
-		}
-	putc('\n', out);
-	echelle /= 10;
-	}
-}
-
-
 #if !(defined(WIN32) || defined(__APPLE__))
 static char seaview_prog_dir[200] = "";
 void inform_prog_dir(const char *arg0)
@@ -1757,6 +1557,7 @@ Fl_Native_File_Chooser *chooser = new Fl_Native_File_Chooser();
 #endif
 
 chooser->type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);   
+  chooser->options(Fl_Native_File_Chooser::SAVEAS_CONFIRM);
 chooser->title(message);  
 chooser->preset_file(fname); 
 	
@@ -1791,3 +1592,326 @@ const char *extract_dirname(const char *pathname)
 		}
 	return dirname;
 }
+
+int printout(SEA_VIEW *view, const char *filename, 
+	     int fontsize, int block_size, Fl_Paged_Device::Page_Format pageformat, int vary_only, int ref0, int pdfkindvalue)
+{
+  int num, i, j, k, current, max_seq_length, fin, curr_lines, widnames, 
+  res_per_line, nl, firstpage, lines_per_page, use_pdf;
+  Fl_PDF_or_PS_File_Device *surface;
+  FILE *textfile = NULL;
+  time_t heure;
+  static char unnamed[] = "<unnamed>";
+  static char num_line[200];
+  int lettre, char_per_line;
+  short *vary_need = NULL;
+  int *vary_pos; /* rang ds alignement de la colonne imprime */
+  char oneline[500];
+  int (*calc_color_function)(int);
+  double	char_width, descender, margin = 25;
+  if(view->tot_seqs == 0) return 0;
+  if(view->protein) calc_color_function = get_color_for_aa;
+  else  calc_color_function = get_color_for_base;
+  use_pdf = (pdfkindvalue != TEXT_ONLY);
+  if(use_pdf) {
+    surface = new Fl_PDF_or_PS_File_Device();
+    if (surface->begin_document(filename, pageformat)) {
+      delete surface;
+      return 0;
+    }
+  }
+  else {
+    textfile = fopen(filename, "w");
+    if(textfile == NULL) return TRUE;
+  }
+#ifndef NO_PDF
+  jmp_buf* jbuf;
+  if (use_pdf) surface->surface_try(&jbuf);
+  if ( (!use_pdf) || (setjmp(*jbuf) == 0)) {
+#endif
+    if(use_pdf) {
+      surface->driver()->font(FL_COURIER, fontsize);
+      char_width = fl_width("X");
+      int pwidth, pheight;
+      surface->printable_rect(&pwidth, &pheight);
+      int l, r, t, b;
+      surface->margins(&l, &t, &r, &b);
+      margin -= l;
+      char_per_line = (int)((pwidth - 2*margin) / char_width + 0.5);
+      lines_per_page = (int)((pheight - 2*margin) / fontsize + 0.5);
+      descender = fl_descent();
+    }
+    else char_per_line = 90;
+    firstpage = TRUE;
+    
+    if(ref0 < 0) vary_only = FALSE;
+    time(&heure);
+    sprintf(oneline,"Alignment: %s", view->masename == NULL ? unnamed : PREPARE_LABEL(view->masename) );
+    if(use_pdf) {
+      surface->start_page();
+      surface->origin(0, fontsize);
+      surface->driver()->font(FL_COURIER, fontsize);
+      fl_draw(oneline, margin, margin); 
+    }
+    else {fputs(oneline, textfile); fputs("\n", textfile);}
+    curr_lines = 1;
+    if(vary_only) {
+      const char fixed[] = "Displaying variable sites only.";
+      if(use_pdf) fl_draw(fixed, margin, margin + curr_lines * fontsize); 
+      else {fputs(fixed, textfile); fputs("\n", textfile);}
+      ++curr_lines;
+    }
+    sprintf(oneline,"Seaview [blocks=%d fontsize=%d %s] on %s",
+	    block_size, fontsize, pageformat == Fl_Paged_Device::A4 ? "A4" : "LETTER", ctime(&heure));
+    if(use_pdf) {
+      char *p = strchr(oneline, '\n'); if (p) *p = 0;
+      fl_draw(oneline, margin, margin + curr_lines * fontsize); 
+    }
+    else {
+      fputs("Seaview text-only output\n", textfile);
+    }
+    curr_lines += 2;
+    max_seq_length = 0; widnames = 0;
+    for(i=0; i < view->tot_seqs; i++) {
+      if(view->each_length[i] > max_seq_length) max_seq_length = view->each_length[i];
+      if( ( fin=strlen(view->seqname[i]) ) > widnames) widnames = fin;
+    }
+    widnames += 2;
+    if(vary_only) {
+      vary_need = (short *)calloc(max_seq_length, sizeof(short));
+      if(vary_need == NULL) return TRUE;
+      vary_pos = (int *)calloc(char_per_line, sizeof(int));
+      if(vary_pos == NULL) return TRUE;
+      for(i = 0; i < max_seq_length; i++) {
+	for(num = 0; num < view->tot_seqs; num++) {
+	  if( toupper(view->sequence[num][i]) != toupper(view->sequence[ref0][i]) ) { 
+	    vary_need[i] = TRUE;
+	    break;
+	  }
+	}
+      }
+    }
+    /* nombre max de blocks qui tiennent sur une ligne de cpl chars */
+    fin = (char_per_line - widnames + 1) / (block_size + 1);
+    if(fin < 1) { /* garde fou */
+      fin = 1; block_size = char_per_line - widnames;
+    }
+    res_per_line = fin * block_size;
+    current = 0; 
+    while( current < max_seq_length ) {
+      nl = 1;
+      if(vary_only) { 
+	memset(vary_pos, 0, res_per_line * sizeof(int) );
+	i = -1; j = 0; k = 0;
+	while( j < res_per_line) {
+	  if(current + i >= max_seq_length) break;
+	  if( !vary_need[current + ++i] ) continue;
+	  j++;
+	  vary_pos[k++] = current + i + 1;
+	  if( j % block_size == 0) k++;
+	}
+	nl = calc_vary_lines(vary_pos,  k);
+      }
+      if( use_pdf && (!firstpage) && (curr_lines + view->tot_seqs + nl > lines_per_page)) {
+	surface->end_page();
+	surface->start_page();
+	surface->origin(0, fontsize);
+	surface->driver()->font(FL_COURIER, fontsize);
+	curr_lines = 0;
+      }
+      if(vary_only) {
+	out_vary_pos(vary_pos, widnames, k, nl, textfile, margin, margin + curr_lines * fontsize);
+	curr_lines += nl;
+      }
+      else	{
+	sprintf(num_line, "%d", current + 1);
+	fin = strlen(num_line);
+	memmove(num_line + widnames - fin + 1, num_line, fin+1);
+	if(fin <= widnames) memset(num_line, ' ', widnames - fin + 1);
+	if( use_pdf) fl_draw(num_line, margin, margin + curr_lines * fontsize); 
+	else {fputs(num_line, textfile);fputs("\n",textfile);}
+	++curr_lines;
+      }
+      for(num=0; num < view->tot_seqs; num++) {
+	k = 0;
+	for(j = 0; j < widnames; j++) {
+	  if(view->seqname[num][j] == 0) break;
+	  oneline[k++] = view->seqname[num][j];
+	}
+	while( j < widnames) {
+	  j++;
+	  oneline[k++] = ' ';
+	}
+	if(vary_only) {
+	  i = -1; j = 0;
+	  while( j < res_per_line) {
+	    if(current + i >= max_seq_length) break;
+	    if( !vary_need[current + ++i] ) continue;
+	    j++;
+	    if(current + i < view->each_length[num]) {
+	      if(num != ref0) lettre = ( toupper(view->sequence[num][current+i]) == 
+					toupper(view->sequence[ref0][current+i]) ? '.' : view->sequence[num][current+i] );
+	      else lettre = view->sequence[ref0][current+i];
+	      oneline[k++] = lettre;
+	    }
+	    if( j % block_size == 0) oneline[k++] = ' ';
+	  }
+	  if(num == view->tot_seqs - 1) current = current + i + 1;
+	}
+	
+	else	{
+	  fin = res_per_line;
+	  if(current+fin > view->each_length[num]) 
+	    fin = view->each_length[num] - current;
+	  if(ref0 != -1 && num != ref0) {
+	    /* ecriture par reference a seq ref0 */
+	    for(i=0; i<fin; i++) {
+	      lettre = ( toupper(view->sequence[num][current+i]) == 
+			toupper(view->sequence[ref0][current+i]) ? '.' : view->sequence[num][current+i] );
+	      oneline[k++] = lettre;
+	      if( i < fin-1 && (i+1)%block_size == 0) 
+		oneline[k++] = ' ';
+	    }
+	  }
+	  else	{ /* ecriture normale de seq */
+	    for(i=0; i<fin; i++) {
+	      oneline[k++] = view->sequence[num][current+i];
+	      if( i < fin-1 && (i+1)%block_size == 0) 
+		oneline[k++] = ' ';
+	    }
+	  }
+	}
+	oneline[k] = 0;
+	if(!view->allow_lower) majuscules(oneline + widnames);
+	if(use_pdf && (curr_lines >= lines_per_page)) {
+	  surface->end_page();
+	  surface->start_page();
+	  surface->origin(0, fontsize);
+	  surface->driver()->font(FL_COURIER, fontsize);
+	  curr_lines = 0;
+	}
+	if(!use_pdf) {
+	  fputs(oneline, textfile); fputs("\n", textfile);
+	}
+	else if(pdfkindvalue == PDF_BW) {
+	  fl_draw(oneline, margin, margin + curr_lines * fontsize);
+	}
+	else {
+	  color_pdf_display(view, calc_color_function, oneline, widnames, margin, 
+			    margin + curr_lines * fontsize, 
+			    fontsize, char_width, descender, num, current);
+	}
+	++curr_lines;
+	firstpage = FALSE;
+      }
+      if(!use_pdf) {
+	fputs("\n", textfile);
+      }
+      else if(curr_lines + 1 <= lines_per_page) {
+	++curr_lines;
+      }
+      if( ! vary_only ) current += res_per_line;
+    }
+    if(use_pdf) {
+      surface->end_page();
+      surface->end_job();
+    }
+    else fclose(textfile);
+#ifndef NO_PDF
+  } /* end of PDF_TRY */
+  if (use_pdf && surface->surface_catch()) {
+    surface->error_catch();
+  }
+#endif
+  if (use_pdf) delete surface;
+  return FALSE;
+}
+
+
+static void color_pdf_display(SEA_VIEW *view, int (*calc_color_function)( int ), char *oneline, 
+			      int widnames, double x, double y, int fontsize, double char_width, double descender,
+			      int num, int current)
+{
+  double  xx;
+  int c, l, count = 0;
+  char *p, **clines;
+  
+  clines = (char **)malloc(sizeof(char *) * view->numb_gc); if(clines==NULL) return;
+  l = strlen(oneline);
+  for(c = 1; c < view->numb_gc; c++) {
+    clines[c] = (char *)malloc(l + 1); if(clines[c] == NULL) return;
+    memset(clines[c], ' ', l); clines[c][l] = 0;
+  }
+  for(p = oneline + widnames; *p != 0; p++) {
+    if(*p == ' ') continue;
+    if(view->curr_colors != view->codoncolors) c = calc_color_function(*p);
+    else c = view->col_rank[num][current + count++];
+    if(c > 0) clines[c][p - oneline] = 'X';
+  }
+  for(c = 1; c < view->numb_gc; c++) {
+    if(strchr(clines[c], 'X') == NULL) continue;
+    fl_color(view->curr_colors[c]);
+    for(xx = x + widnames*char_width, p = clines[c] + widnames; *p != 0; p++, xx += char_width) {
+      if(*p == ' ') continue;
+      fl_rectf(xx, y - fontsize + descender, char_width, fontsize);
+    }
+  }
+  fl_color(FL_BLACK);
+  fl_draw(oneline, x, y);
+  for(c = 1; c < view->numb_gc; c++) free(clines[c]);
+  free(clines);
+}
+
+
+static int calc_vary_lines(int *vary_pos, int widpos)
+{
+  int maxi = 0, num, nl;
+  
+  for(num = 0; num < widpos; num++) 
+    if(vary_pos[num] > maxi) maxi = vary_pos[num];
+  if(maxi >= 100000)
+    nl = 6;
+  else if(maxi >= 10000)
+    nl = 5;
+  else if(maxi >= 1000)
+    nl = 4;
+  else if(maxi >= 100)
+    nl = 3;
+  else if(maxi >= 10)
+    nl = 2;
+  else 	
+    nl = 1;
+  return nl;
+}
+
+
+static void out_vary_pos(int *vary_pos, int widnames, int widpos, int nl, FILE *textfile, double x, double y)
+{
+  int num, l, k, echelle, digit, val;
+  static char chiffre[] = "0123456789";
+  char oneline[300];
+  
+  echelle = 1; k = 0;
+  for(l = 2; l <= nl; l++) echelle *= 10;
+  for(l = nl; l > 0; l--) {
+    for(num = 0; num < widnames; num++) oneline[k++] = ' ';
+    for(num = 0; num < widpos; num++) {
+      val = vary_pos[num];
+      if(val < echelle)
+	oneline[k++] = ' ';
+      else	{
+	digit = (val / echelle) % 10 ;
+	oneline[k++] = *(chiffre + digit);
+      }
+    }
+    oneline[k] = 0;
+    if(textfile == NULL) {
+      fl_draw(oneline, x, y);
+      y += fl_height();
+    }
+    else {fputs(oneline, textfile); fputs("\n",textfile); }
+    k = 0;
+    echelle /= 10;
+  }
+}
+
