@@ -17,7 +17,6 @@ extern "C" {
 	char *get_code_descr(int code);
 	}
 extern int totcodes;
-extern void get_menu_taille_valeur(Fl_Widget *menu, int *taille, int *valeur);
 extern void set_and_show_new_cursor_seq(SEA_VIEW *view, int new_pos);
 extern void set_and_show_new_cursor_site(SEA_VIEW *view, int new_pos, int center,
 	int force_redraw);
@@ -25,7 +24,7 @@ extern void set_and_show_new_cursor_site(SEA_VIEW *view, int new_pos, int center
 
 void *set_viewasprots(SEA_VIEW * view, int onoff)
 {
-int num, oldmodif, gc, oldtotsel, *oldselseqs, oldsites, taille, oldcursorseq, oldcursorsite;
+int num, oldmodif, gc, oldtotsel, *oldselseqs, oldsites, oldcursorseq, oldcursorsite;
 char **protseq, **dnaseq, **olddna, **retval, *masename;
 
 oldmodif = view->modif_but_not_saved;
@@ -35,7 +34,7 @@ oldcursorseq = view->cursor_seq;
 oldcursorsite = view->cursor_site;
 save_active_region(view);
 masename = view->masename;
-get_menu_taille_valeur( (Fl_Menu_ *)(view->menu_regions), &taille, &oldsites);
+  oldsites = view->menu_sites->value();
 if(onoff) { /* ON */
 	protseq = (char **)malloc(view->tot_seqs * sizeof(char *));
 	if(protseq == NULL) return NULL;
@@ -67,7 +66,7 @@ if(onoff) { /* ON */
 	      lseg = lseg->next;
 	      }
 	    if(lreg->element->hide_when_viewasprots) { // inactivate in menu if hide_when_viewasprots is on
-	      ((Fl_Menu_Button*)view->menu_regions)->mode(4 + i - 1, FL_MENU_RADIO | FL_MENU_INACTIVE);
+	      view->menu_sites->vitem(i - 1)->deactivate();
 	      if(i == oldsites) oldsites = 0; // and don't show it as active region
 	    }
 	    lreg = lreg->next;
@@ -130,8 +129,9 @@ free(view->sel_seqs);
 free(masename);
 view->sel_seqs = oldselseqs;
 if(oldsites) {
-  ((Fl_Menu_ *)(view->menu_regions))->value(oldsites + 3);
-  ((Fl_Menu_ *)(view->menu_regions))->do_callback();
+  view->menu_sites->vitem(oldsites-1)->set();
+  view->menubar->value(view->menu_sites->vitem(oldsites-1));
+  view->menu_sites->vitem(oldsites-1)->callback()(view->menubar, 0);
   }
 select_deselect_seq(view, -2);
 view->DNA_obj->redraw();
