@@ -1,28 +1,15 @@
 #ifdef WIN32
-#include "treedraw.h"
+#include "pdf_or_ps.h"
 #include <Windows.h>
-#include <FL/Fl.H>
-#include "FL/fl_ask.H"
-#include "FL/x.H"
-#include "FL/fl_draw.H"
-#include "src/Fl_Font.H"
-#include <stdarg.h>
-#include <string.h>
-#include <direct.h>
+#include <FL/fl_utf8.h>
 #include <stdlib.h>
 #include <stdio.h>
-#ifndef NO_PDF
 #include <pdflib.h>
-#endif
 
 /* included functions */
 void my_bell(void);
-int mysystem(const char *command);
 char *get_prog_dir(void);
 char **getutf8args(int *pargc);
-#if !(FL_MAJOR_VERSION >= 1 && FL_MINOR_VERSION >= 3)
-void draw_rotated_string(const char *strDraw, int x, int y, float degrees /*clockwise*/);
-#endif
 
 /* extern vars & functions */
 extern char *wchartoutf8(const wchar_t *in);
@@ -30,8 +17,18 @@ extern char *wchartoutf8(const wchar_t *in);
 Fl_Copy_Surface::Fl_Copy_Surface(int w, int h) : Fl_Surface_Device(new Fl_GDI_Graphics_Driver())
 {
   HDC oldflgc = fl_gc;
-  int factor = 32; // empirically set
-  RECT rect; rect.left = 0; rect.top = 0; rect.right=w*factor; rect.bottom=h*factor;
+  //int factor = 32; // empirically set
+  // exact computation of factor from screen units to EnhMetaFile units (0.01 mm)
+  HDC hdc = GetDC(NULL);
+  int hmm = GetDeviceCaps(hdc, HORZSIZE);
+  int hdots = GetDeviceCaps(hdc, HORZRES);
+  int vmm = GetDeviceCaps(hdc, VERTSIZE);
+  int vdots = GetDeviceCaps(hdc, VERTRES);
+  ReleaseDC(NULL, hdc);
+  float factorw =  (100. * hmm) / hdots;
+  float factorh =  (100. * vmm) / vdots + 0.5;
+  
+  RECT rect; rect.left = 0; rect.top = 0; rect.right = w * factorw; rect.bottom = h * factorh;
   fl_gc = CreateEnhMetaFile (NULL, NULL, &rect, NULL);
   if (fl_gc != NULL) {
     SetTextAlign(fl_gc, TA_BASELINE|TA_LEFT);
@@ -60,31 +57,6 @@ Fl_Copy_Surface::~Fl_Copy_Surface()
 }
 
 
-int mysystem(const char *command)
-/*
- command contains "cmd.exe /c prog.exe args"  or  "prog.exe args"
- the first form is compulsory if args contain "> outfile"
- prog is searched in dir where calling program was launched, current dir,
- windows system dir, windows dir, dirs in PATH.
- returns 0 iff OK
- */
-{
-	static char buf[100];
-	STARTUPINFO info;
-	PROCESS_INFORMATION pi;
-	int retval;
-	
-	GetStartupInfo(&info);
-	retval = CreateProcess(NULL, (char *)command,0,0,0,0,0,_getcwd(buf,sizeof(buf)),&info,&pi);
-	if(retval == 0) return 1;
-	do	{
-		Fl::check();
-		retval = WaitForSingleObject(pi.hProcess, 500);
-	}
-	while(retval == WAIT_TIMEOUT);
-	return 0;
-}
-
 char *get_prog_dir(void)
 /* returns path of dir where calling program was launched */
 {
@@ -129,67 +101,56 @@ char **getutf8args(int *pargc)
 }
 
 
-#if !(FL_MAJOR_VERSION >= 1 && FL_MINOR_VERSION >= 3)
-void draw_rotated_string(const char *strDraw, int x, int y, float degrees /*clockwise*/)
+const char *win32_UTF8_to_CP1252(const char *in)
 {
-	int n = strlen(strDraw);
-	WCHAR *wout = new WCHAR[n+5];
-	int newl = MultiByteToWideChar(CP_UTF8, 0, strDraw, n, wout, n + 5);
-	LOGFONT lf;
-	memset(&lf, 0, sizeof(LOGFONT));
-	int rank = fl_font() / 4;
-	SelectObject(fl_gc, fl_fontsize->fid);//necessary to put selected fltk's font in fl_gc
-	GetTextFace(fl_gc, sizeof(lf.lfFaceName), lf.lfFaceName);
-
-	if(fl_font() < 12) {//only the first 12 fonts have bold/italic
-		rank = fl_font() % 4;
-		lf.lfWeight = (rank == 1 || rank == 3) ? FW_BOLD : FW_NORMAL;
-		lf.lfItalic = (rank == 2 || rank == 3) ? 1 : 0;
-	}
-	lf.lfHeight = - fl_size();
-
-	// Specify the angle to draw line
-	lf.lfEscapement = - (LONG)(degrees*10);
-	lf.lfOrientation = lf.lfEscapement;
-	// Select the new font created
-	HFONT hFont = (HFONT) CreateFontIndirect(&lf);
-	HFONT hPrevFont = (HFONT) SelectObject(fl_gc, hFont);
-
-	// Draw text to GC with color
-	SetTextColor(fl_gc, fl_RGB());
-	TextOutW(fl_gc, x, y, wout, newl);
-	SelectObject(fl_gc, hPrevFont);
-	DeleteObject(hFont);
-	delete wout;
+  static char dst[250];
+  fl_utf8toa(in, strlen(in), dst, sizeof(dst));
+  return (const char *)dst;
 }
-#endif
 
 extern LPCWSTR utf8towchar(const char *in);
-static FILE *pdf_output = NULL;
 
 static size_t pdf_write_proc(PDF *p, void *data, size_t size)
 {
-  return fwrite(data, 1, size, pdf_output);
+  FILE **output = (FILE**)PDF_get_opaque(p);
+  return fwrite(data, 1, size, *output);
 }
 
-#ifdef PDF_begin_document
-#undef PDF_begin_document 
-#undef PDF_end_document 
-#endif
 // because PDF_begin_document fails on WIN32 if non ascii filename
+
+PDF *win32_PDF_new()
+{
+  FILE **output = new FILE*;
+  PDF *pdf = PDF_new2(0,0,0,0, output);
+  if (!pdf) delete output;
+  return pdf;
+}
+
 int win32_PDF_begin_document(PDF *p, const char *filename, int len, const char *optlist)
 {
-  pdf_output = _wfopen(utf8towchar(filename), L"wb"); // binary mode compulsory
-  if(pdf_output == NULL) return -1;
+  FILE **output = (FILE**)PDF_get_opaque(p);
+  *output = _wfopen(utf8towchar(filename), L"wb"); // binary mode compulsory
+  if (*output == NULL) return -1;
   PDF_begin_document_callback(p, pdf_write_proc, optlist);
   return 1;
 }
 
 void win32_PDF_end_document(PDF *p, const char *optlist)
 {
+  FILE **output = (FILE**)PDF_get_opaque(p);
+#ifdef PDF_end_document
+#undef PDF_end_document 
+#undef PDF_delete 
+#endif
   PDF_end_document(p, optlist);
-  fclose(pdf_output);
-  pdf_output = NULL;
+  fclose(*output);
+}
+
+void win32_PDF_delete(PDF *p)
+{
+  FILE **output = (FILE**)PDF_get_opaque(p);
+  delete output;
+  PDF_delete(p);
 }
 
 /*
@@ -245,6 +206,32 @@ void win32_PDF_end_document(PDF *p, const char *optlist)
  }
  return argv;
  }
+ 
+ int mysystem(const char *command)
+ /*
+ command contains "cmd.exe /c prog.exe args"  or  "prog.exe args"
+ the first form is compulsory if args contain "> outfile"
+ prog is searched in dir where calling program was launched, current dir,
+ windows system dir, windows dir, dirs in PATH.
+ returns 0 iff OK
+ *\
+{
+  static char buf[100];
+  STARTUPINFO info;
+  PROCESS_INFORMATION pi;
+  int retval;
+  
+  GetStartupInfo(&info);
+  retval = CreateProcess(NULL, (char *)command,0,0,0,0,0,_getcwd(buf,sizeof(buf)),&info,&pi);
+  if(retval == 0) return 1;
+  do	{
+    Fl::check();
+    retval = WaitForSingleObject(pi.hProcess, 500);
+  }
+  while(retval == WAIT_TIMEOUT);
+  return 0;
+}
+
  */
 
 #endif // WIN32
